@@ -5,6 +5,20 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
    -- Revisions -----
      Logfile:  %F%
     Revision:  %I%
@@ -39,63 +53,234 @@
 #endif
 
 PSTR vrgszLexTypes[] = {
-  "null-type??",    /* 00 */
-  "identifier",        /* 01 */
-  "integer",        /* 02 */
-  "integer-or-id",    /* 03 */
-  "floating-point",    /* 04 */
-  "float-or-id",    /* 05 */
-  "number",        /* 06 */
-  "number-or-id",    /* 07 */
-  "punctuation",    /* 08 */
-  "invalid lexical type",    /* 09 */
-  "invalid lexical type",    /* 0a */
-  "invalid lexical type",    /* 0b */
-  "invalid lexical type",    /* 0c */
-  "invalid lexical type",    /* 0d */
-  "invalid lexical type",    /* 0e */
-  "invalid lexical type",    /* 0f */
-  "quoted-string",    /* 10 */
-
-  "" /* End flag */
+  "null-type??",           /* 00 */
+  "identifier",            /* 01 */
+  "integer",               /* 02 */
+  "integer-or-id",         /* 03 */
+  "floating-point",        /* 04 */
+  "float-or-id",           /* 05 */
+  "number",                /* 06 */
+  "number-or-id",          /* 07 */
+  "punctuation",           /* 08 */
+  "invalid lexical type",  /* 09 */
+  "invalid lexical type",  /* 0a */
+  "invalid lexical type",  /* 0b */
+  "invalid lexical type",  /* 0c */
+  "invalid lexical type",  /* 0d */
+  "invalid lexical type",  /* 0e */
+  "invalid lexical type",  /* 0f */
+  "quoted-string",         /* 10 */
+  ""                       /* End flag */
 }; /* vrgszLexTypes[] = */
 
 
 /* routines */
 
-/* -----------------------------------------------------------------------------
-   PreventLexSplit
 
-   Prevents the buffer from splitting a lexical element by
-   "backing up" the EOB pointer and adjusting the file pointer
-   to just after a carriage return.
+/* ---------------------------------------------------------------------------
+   ENextLex
+
+   Sames as NextLex but looks for iType.  Reports errors.  Returns
+   0 if ok, non-zero if error.
 */
 
-void PreventLexSplit (PINPUTBUF pibIn, int iOffset)
+BOOL ENextLex (PINPUTBUF pibIn, PSTRLEX szLex, int iType)
 {
-  long lFilePos, lDelta;
-  PBUF pbufEOB = pibIn->pbufOrg + iOffset;
-  PBUF pbufEOBOld;
+  int iLex, iErr;
 
-  if (!EOB(pibIn)        /* If not EOB, use all of input, otherwise... */
-      || (iOffset == BUFFER_SIZE)) {    /* No room for NULL */
+  NextLex (pibIn, szLex, &iLex);
 
-    pbufEOBOld = pbufEOB;        /* Save EOB */
-    lFilePos = ftell (pibIn->pfileIn);    /* Save position */
-    while (*(--pbufEOB) != CH_EOLN)
-      ;                /* Move EOB to last EOLN */
-    *pbufEOB = '\0';        /* Overwrite EOLN with NULL */
+  if ((iErr = !(iType & iLex)))
+    ReportError (pibIn, RE_LEXEXPECTED, vrgszLexTypes[iType], szLex);
 
-    if ((lDelta = (long) (pbufEOB - pbufEOBOld)))
-      fseek (pibIn->pfileIn, lDelta, SEEK_CUR);    /* Backup file ptr */
-  } /* if */
+  return (iErr);
 
-  else *pbufEOB = '\0'; /* Append NULL */
-
-} /* PreventLexSplit */
+} /* ENextLex */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
+   EvalAtom
+
+   Evaluate an atomic expression.
+*/
+
+long EvalAtom (PINPUTBUF pibIn, long index, PSTR *szExp, PSTR szToken, 
+               PINT piType)
+{
+  long result = 0;
+
+  switch (*piType) {
+    case LX_IDENTIFIER:
+      result = index;
+      GetToken (szExp, szToken, piType);
+      break;
+
+    case LX_INTEGER:
+      result = atol (szToken);
+      GetToken (szExp, szToken, piType);
+      break;
+
+    default:
+      ReportError (pibIn, RE_UNEXPECTED | RE_FATAL, *szExp, 
+                   "(While parsing bracketed expression)");
+  }
+
+  return (result);
+
+} /* EvalAtom */
+
+
+/* ---------------------------------------------------------------------------
+   EvalParen
+
+   Evaluate a parenthetized expression.
+*/
+
+long EvalParen (PINPUTBUF pibIn, long index, PSTR *szExp, PSTR szToken, 
+                PINT piType)
+{
+  long result;
+
+  if (*szToken == '(') { 
+    GetToken (szExp, szToken, piType);
+    result = EvalSum (pibIn, index, szExp, szToken, piType);
+    if (*szToken != ')')
+      ReportError (pibIn, RE_UNEXPECTED | RE_FATAL, *szExp, 
+                   "(While parsing bracketed expression)");
+    GetToken (szExp, szToken, piType);
+  }
+  else
+    result = EvalAtom (pibIn, index, szExp, szToken, piType);
+
+  return (result);
+
+} /* EvalParen */
+
+
+/* ---------------------------------------------------------------------------
+   EvalProd
+
+   Multiply or divide two terms in an expression.
+*/
+
+long EvalProd (PINPUTBUF pibIn, long index, PSTR *szExp, PSTR szToken, 
+               PINT piType)
+{
+  register char cOperator;
+  long result, dTmp;
+
+  result = EvalUnary (pibIn, index, szExp, szToken, piType);
+
+  while (((cOperator = *szToken) == '*') || (cOperator == '/')) {
+    GetToken (szExp, szToken, piType);
+    dTmp = EvalUnary (pibIn, index, szExp, szToken, piType);
+    switch (cOperator) {
+      case '*':
+        result = result * dTmp;
+        return (result);
+
+      case '/':
+        result = result / dTmp;
+        return (result);
+
+      default:
+        ReportError (pibIn, RE_UNEXPECTED | RE_FATAL, *szExp, 
+                     "(While parsing bracketed expression)");
+    }
+  }
+  return (result);
+
+} /* EvalProd */
+
+
+/* ---------------------------------------------------------------------------
+   EvalSum
+
+   Add or subtract two terms in an expression.
+*/
+
+long EvalSum (PINPUTBUF pibIn, long index, PSTR *szExp, PSTR szToken, 
+              PINT piType)
+{
+  register char cOperator;
+  long result, dTmp;
+
+  result = EvalProd (pibIn, index, szExp, szToken, piType);
+
+  while (((cOperator = *szToken) == '+') || (cOperator == '-')) {
+    GetToken (szExp, szToken, piType);
+    dTmp = EvalProd (pibIn, index, szExp, szToken, piType);
+    switch (cOperator) {
+      case '-':
+        result = result - dTmp;
+        break; 
+
+      case '+':
+        result = result + dTmp;
+        break;
+
+      default:
+        ReportError (pibIn, RE_UNEXPECTED | RE_FATAL, *szExp, 
+                     "(While parsing bracketed expression)");
+    }
+  }
+  return (result);
+
+} /* EvalSum */
+
+
+/* ---------------------------------------------------------------------------
+   EvaluateExpression
+
+   return the value of a simple expression. Expressions can be composed of 
+   integers, the 4 basic arithmetic operators, parenthese and 'i'.
+*/
+
+long EvaluateExpression (PINPUTBUF pibIn, long index, PSTR szExpress)
+{
+  PSTRLEX szToken;
+  int iType;
+
+  GetToken (&szExpress, szToken, &iType);
+  if (!*szToken) {
+    ReportError (pibIn, RE_UNEXPECTED | RE_FATAL, szExpress, 
+                 "(While parsing bracketed expression)");
+    return (0);
+  }
+  else 
+    return EvalSum (pibIn, index, &szExpress, szToken, &iType);
+
+} /* EvaluateExpression */
+
+
+/* ---------------------------------------------------------------------------
+   EvalUnary
+
+   Evaluate a unary + or - in an expression.
+*/
+
+long EvalUnary (PINPUTBUF pibIn, long index, PSTR *szExp, PSTR szToken, 
+                PINT piType)
+{
+  register char cOperator;
+  long result;
+
+  cOperator = 0;
+
+  if ((*piType == LX_EQNPUNCT) && ((*szToken == '+') || (*szToken == '-'))) {
+    cOperator = *szToken;
+    GetToken (szExp, szToken, piType);
+  }
+  result = EvalParen (pibIn, index, szExp, szToken, piType);
+  if (cOperator == '-')
+    result = -result;
+
+  return (result);
+
+} /* EvalUnary */
+
+
+/* ---------------------------------------------------------------------------
    FillBuffer
 
    Fills the initialized input buffer and sets all buffer pointers.
@@ -128,7 +313,53 @@ int FillBuffer (PINPUTBUF pibIn)
 } /* FillBuffer */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
+   GetToken
+
+*/
+
+void GetToken (PSTR *szExp, PSTR szToken, PINT piType)
+{
+  register PSTR cTmp;
+
+  *piType = 0;
+  cTmp = szToken;
+  *cTmp = '\0';
+
+  if (!(*szExp)) return;
+
+  while (isspace (**szExp)) /* skip white space */
+    (*szExp)++;
+
+  if (strchr ("+-*/()", **szExp)) {
+    *piType = LX_EQNPUNCT;
+    *cTmp = **szExp; /* copy */
+    cTmp++; /* advance */
+    (*szExp)++; /* advance */
+  }
+  else if (**szExp == 'i') {
+    *piType = LX_IDENTIFIER;
+    while (!(strchr ("+-*/()", **szExp) || (**szExp == '\0'))) {
+      *cTmp = **szExp; /* copy */
+      cTmp++; /* advance */
+      (*szExp)++; /* advance */
+    }
+  }
+  else if (isdigit (**szExp)) {
+    *piType = LX_INTEGER;
+    while (!(strchr("+-*/()", **szExp) || (**szExp == '\0'))) {
+      *cTmp = **szExp; /* copy */
+      cTmp++; /* advance */
+      (*szExp)++; /* advance */
+    }
+  }
+
+  *cTmp = '\0'; /* terminate */
+
+} /* GetToken */
+
+
+/* ---------------------------------------------------------------------------
    InitBuffer
 
    Initializes the input buffer whose address is pibIn with the
@@ -151,20 +382,21 @@ BOOL InitBuffer (PINPUTBUF pibIn, PSTR szFullPathname)
   pibIn->pInfo = NULL;
   pibIn->pbufCur = NULL;
 
-  if ((pibIn->pfileIn = fopen (szFullPathname, "r"))
-      && (pibIn->pbufOrg = (PBUF) malloc (BUFFER_SIZE))) {
-    bReturn = FillBuffer (pibIn);
-  } /* if */
+  if ((pibIn->pfileIn = fopen (szFullPathname, "r"))) {
+    if ((pibIn->pbufOrg = (PBUF) malloc (BUFFER_SIZE)))
+      bReturn = FillBuffer (pibIn);
+    else
+      ReportError (pibIn, RE_OUTOFMEM | RE_FATAL, "InitBuffer", NULL);
+  }
   else
-    ReportError (pibIn,
-                 RE_FILENOTFOUND | RE_FATAL, szFullPathname, NULL);
+    ReportError (pibIn, RE_FILENOTFOUND | RE_FATAL, szFullPathname, NULL);
 
   return (bReturn);
 
 } /* InitBuffer */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
    MakeStringBuffer
 
    Makes a string buffer from a string.
@@ -193,56 +425,67 @@ void FlushBuffer (PINPUTBUF pibIn)
   while (*pbuf)
     printf ("%c", *pbuf++);
   printf ("");
+
 } /* FlushBuffer */
 
 
-/* -----------------------------------------------------------------------------
-   SkipWhitespace
-
-   Skips over whitespace of input buffer.  Returns non-zero if something
-   has been skipped.
+/* ---------------------------------------------------------------------------
+   GetArrayBounds
+   return the lower (LB) upper (UB) bounds of an array given between []. 
+   GetArrayBounds must be called after finding [. It reads up to ], included.
+   Errors are generated if LB < 0 or UB < LB. 
+   Syntax for bounds:
+   [i]:   bounds returned are i to i+1
+   [i-j]: bounds returned are i to j+1
+   where i and j are long integers
 */
 
-int SkipWhitespace (PINPUTBUF pibIn)
+void GetArrayBounds (PINPUTBUF pibIn, PLONG piLB, PLONG piUB)
 {
-  char c;
-  int fSkipped = 0;
+  PSTRLEX szTmp;
 
-  if (!pibIn)
-    return 0;
-
-  if (!*pibIn->pbufCur && pibIn->pfileIn)
-    FillBuffer (pibIn);
-
-  /* Skip Spaces, Tabs and Newlines */
-
-  while (isspace(c = *pibIn->pbufCur) || c == CH_COMMENT) {
-    fSkipped = 1;
-    if (IsComment(&c))
-      SkipComment (pibIn);
+  if (ENextLex (pibIn, szTmp, LX_INTEGER)) {
+    ReportError (pibIn, RE_INIT | RE_FATAL, NULL, NULL);
+  }
+  else {
+    *piLB = atol(szTmp);
+    if (*piLB < 0) 
+      ReportError (pibIn, RE_POSITIVE | RE_FATAL, szTmp, NULL);
 
+    if (NextChar (pibIn) == '-') { /* get eventual hyphen */
+      pibIn->pbufCur++; /* advance */
+      if (ENextLex (pibIn, szTmp, LX_INTEGER)) {
+        ReportError (pibIn, RE_INIT | RE_FATAL, NULL, NULL);
+      }
+      else {
+      *piUB = atol(szTmp) + 1;
+        if (*piUB <= *piLB) 
+         ReportError (pibIn, RE_UNKNOWN | RE_FATAL, "", 
+                      "Upper bound must be higher than lower bound");
+      }
+      if (!GetPunct (pibIn, szTmp, ']')) { /* get closing bracket */
+        ReportError (pibIn, RE_LEXEXPECTED | RE_FATAL, "]", NULL);
+      }
+    }
     else {
-      if (c == '\n')
-    pibIn->iLineNum++;
-
-      if (!*(++pibIn->pbufCur) && pibIn->pfileIn)
-    if (FillBuffer (pibIn) == EOF)
-      break;
-    } /* else */
-  } /* while */
-
-  return (fSkipped);
-
-} /* SkipWhitespace */
+      if (!GetPunct (pibIn, szTmp, ']')) { /* get closing bracket */
+        ReportError (pibIn, RE_LEXEXPECTED | RE_FATAL, "]", NULL);
+      }
+      else { /* a number is an index, the upper bound is set at LB+1 */
+        *piUB = *piLB + 1;
+      }
+    }
+  }
+} /* GetArrayBounds */
 
 
-/* -----------------------------------------------------------------------------
-   GetString
+/* ---------------------------------------------------------------------------
+   GetaString
 
    Copies the quoted string from buffer to szLex.
 */
 
-void GetString (PINPUTBUF pibIn, PSTR szLex)
+void GetaString (PINPUTBUF pibIn, PSTR szLex)
 {
   int i = 0;
 
@@ -265,10 +508,68 @@ void GetString (PINPUTBUF pibIn, PSTR szLex)
 
   szLex[i] = '\0'; /* Overwrite closing delim with '\0' */
 
-} /* GetString */
+} /* GetaString */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
+   GetFuncArgs
+
+   Gets nArgs arguments to a "function" from pibIn.  The argument
+   list must be in parentheses.
+
+   rgiArgTypes[] is a profile specifying which type each argument must be.
+   rgszArgs[] is an array of PSTRLEX buffers to hold the arguments.
+   rgiLowerB[] and rgiUpperB[] are arrays of lower and upper bounds. Bounds
+   are set to -1 for scalar variables.
+
+   Returns TRUE on success, FALSE on an error.  Errors are reported,
+   but the statement is not flushed.
+*/
+
+BOOL GetFuncArgs (PINPUTBUF pibIn, int nArgs, int rgiArgTypes[], PSTR szArgs,
+                  long rgiLowerB[], long rgiUpperB[])
+{
+  BOOL bOK = TRUE;
+  int i, iType;
+  PSTRLEX szPunct;
+
+  if (!(bOK = GetPunct (pibIn, szPunct, CH_LPAREN))) {
+    szPunct[1] = CH_LPAREN;
+    ReportError (pibIn, RE_EXPECTED, szPunct, NULL);
+  }
+
+  for (i = 0; i < nArgs && bOK; i++, szArgs += MAX_LEX) {
+    if (i)
+      if (!(bOK = GetOptPunct (pibIn, szArgs, ','))) {
+        *(szArgs+1) = ',';
+        ReportError (pibIn, RE_EXPECTED, szArgs, NULL);
+        break; /* Error: Stop getting args */
+      }
+
+    NextLex (pibIn, szArgs, &iType);
+    if (!(bOK &= (iType & rgiArgTypes[i]) > 0))
+      ReportError (pibIn, RE_LEXEXPECTED, vrgszLexTypes[rgiArgTypes[i]],
+                   szArgs);
+
+    rgiLowerB[i] = rgiUpperB[i] = -1;
+    if (GetPunct (pibIn, szPunct, '[')) /* array found, read bounds */
+      GetArrayBounds (pibIn, &rgiLowerB[i], &rgiUpperB[i]);
+
+  } /* for i */
+
+  /* try to get closing parenthesis if not already gotten */
+  if (!(bOK = (szPunct[0] == CH_RPAREN || 
+               GetPunct (pibIn, szPunct, CH_RPAREN)))) {
+    szPunct[1] = CH_RPAREN;
+    ReportError (pibIn, RE_EXPECTED, szPunct, NULL);
+  }
+
+  return (bOK);
+
+} /* GetFuncArgs */
+
+
+/* ---------------------------------------------------------------------------
    GetIdentifier
 
    Copies identifier from buffer to szLex.  A valid id begins with a
@@ -297,7 +598,67 @@ void GetIdentifier (PINPUTBUF pibIn, PSTR szLex)
 } /* GetIdentifier */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
+   GetInteger
+
+   Gets the next lexical element as an integer number from the input buffer. 
+   piLexType is set accordingly.
+
+   An integer is defined as a sequence of digits.
+
+   Leading signs to the numbers are handled separately as unary
+   or binary operators.
+*/
+
+void GetInteger (PINPUTBUF pibIn, PSTR szLex, PINT piLexType)
+{
+  PBUF pbufCur;
+  int i = 0;
+  char c;
+  BOOL bHasSign = FALSE;
+  BOOL bLeadingDigits = FALSE;
+  enum States
+   {Start, Digits1, End} eState;
+
+  if (!pibIn || !szLex || !piLexType)
+    return;
+
+  pbufCur = pibIn->pbufCur;
+  eState = Start;
+  *piLexType = LX_NULL;
+  while ((c = *pibIn->pbufCur) && i < MAX_LEX-1 && eState != End) {
+
+    switch (eState) {
+      case Start:
+        if (!bHasSign && IsSign(c))
+          bHasSign = TRUE;
+        else if (isdigit(*pibIn->pbufCur)) {
+          bLeadingDigits = *piLexType = LX_INTEGER;
+          eState = Digits1;
+        } /* else */
+        else
+          eState = End;
+        break;
+
+      case Digits1:
+        if (!isdigit(c))
+          eState = End;
+        break;
+
+      case End:
+        break;
+    } /* switch */
+
+    if (eState != End)
+      szLex[i++] = *pibIn->pbufCur++;
+
+  } /* while */
+  szLex[i] = '\0';
+
+} /* GetInteger */
+
+
+/* ---------------------------------------------------------------------------
    GetNumber
 
    Gets the next lexical element as either a floating-point or
@@ -334,88 +695,87 @@ void GetNumber (PINPUTBUF pibIn, PSTR szLex, PINT piLexType)
   pbufCur = pibIn->pbufCur;
   eState = Start;
   *piLexType = LX_NULL;
-  while ((c = *pibIn->pbufCur)
-         && i < MAX_LEX-1
-     && eState != End) {
+  while ((c = *pibIn->pbufCur) && i < MAX_LEX-1 && eState != End) {
 
     switch (eState) {
       case Start:
-    if (c == '.')
-      eState = Point;
-    else if (!bHasSign && IsSign(c))
-      bHasSign = TRUE;
-    else if (isdigit(*pibIn->pbufCur)) {
-      bLeadingDigits = *piLexType = LX_INTEGER;
-      eState = Digits1;
-    } /* else */
-    else
-      eState = End;
-    break;
+        if (c == '.')
+          eState = Point;
+        else if (!bHasSign && IsSign(c))
+          bHasSign = TRUE;
+        else if (isdigit(*pibIn->pbufCur)) {
+          bLeadingDigits = *piLexType = LX_INTEGER;
+          eState = Digits1;
+        } /* else */
+        else
+          eState = End;
+        break;
 
       case Digits1:
-    if (c == '.')
-      eState = Point;
-    else if (c == 'e' || c == 'E')
-      eState = Exp;
-    else if (!isdigit(c))
-      eState = End;
-    break;
+        if (c == '.')
+          eState = Point;
+        else if (c == 'e' || c == 'E')
+          eState = Exp;
+        else if (!isdigit(c))
+          eState = End;
+        break;
 
       case Point:
-    *piLexType = LX_FLOAT;
-    if (bLeadingDigits && (c == 'e' || c == 'E'))
-      eState = Exp;
-    else if (isdigit(c))
-      eState = Digits2;
-    else {
-      if (!bLeadingDigits)        /* Error, point only */
-        *piLexType = LX_NULL;
-      eState = End;
-    } /* else */
-    break;
+        *piLexType = LX_FLOAT;
+        if (bLeadingDigits && (c == 'e' || c == 'E'))
+          eState = Exp;
+        else if (isdigit(c))
+          eState = Digits2;
+        else {
+          if (!bLeadingDigits) /* Error, point only */
+            *piLexType = LX_NULL;
+          eState = End;
+        } /* else */
+        break;
 
       case Digits2:
-    if (c == 'e' || c == 'E')
-      eState = Exp;
-    else if (!isdigit(c))
-      eState = End;
-    break;
+        if (c == 'e' || c == 'E')
+          eState = Exp;
+        else if (!isdigit(c))
+          eState = End;
+        break;
 
       case Exp:
-    *piLexType = LX_FLOAT;
-    if (IsSign(c)) {
-      eState = ExpSign;
-      break;
-    } /* if */
+        *piLexType = LX_FLOAT;
+        if (IsSign(c)) {
+          eState = ExpSign;
+          break;
+        } /* if */
         /* Fall through! */
 
       case ExpSign:
-    if (isdigit(c))
-      eState = Digits3;
-    else {
-      *piLexType = LX_NULL;
-      eState = End;
-    }
-    break;
+        if (isdigit(c))
+          eState = Digits3;
+        else {
+          *piLexType = LX_NULL;
+          eState = End;
+        }
+        break;
 
       case Digits3:
-    if (!isdigit(c))
-      eState = End;
+        if (!isdigit(c))
+          eState = End;
         break;
 
       case End:
-    break;
+        break;
     } /* switch */
 
     if (eState != End)
       szLex[i++] = *pibIn->pbufCur++;
+
   } /* while */
   szLex[i] = '\0';
 
 } /* GetNumber */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
    NextLex
 
    Skips over leading whitespace and copies the next lexical element
@@ -463,7 +823,7 @@ void NextLex (PINPUTBUF pibIn, PSTRLEX szLex, PINT piLexType)
       else
       if (IsString (&c)) {
         *piLexType = LX_STRING;
-        GetString (pibIn, szLex);
+        GetaString (pibIn, szLex);
       } /* if */
 
       else if (strchr(vszEqnPunct, c)) {
@@ -488,28 +848,39 @@ void NextLex (PINPUTBUF pibIn, PSTRLEX szLex, PINT piLexType)
 } /* NextLex */
 
 
-/* -----------------------------------------------------------------------------
-   ENextLex
+/* ---------------------------------------------------------------------------
+   PreventLexSplit
 
-   Sames as NextLex but looks for iType.  Reports errors.  Returns
-   0 if ok, non-zero if error.
+   Prevents the buffer from splitting a lexical element by
+   "backing up" the EOB pointer and adjusting the file pointer
+   to just after a carriage return.
 */
 
-BOOL ENextLex (PINPUTBUF pibIn, PSTRLEX szLex, int iType)
+void PreventLexSplit (PINPUTBUF pibIn, int iOffset)
 {
-  int iLex, iErr;
+  long lFilePos, lDelta;
+  PBUF pbufEOB = pibIn->pbufOrg + iOffset;
+  PBUF pbufEOBOld;
 
-  NextLex (pibIn, szLex, &iLex);
+  if (!EOB(pibIn)        /* If not EOB, use all of input, otherwise... */
+      || (iOffset == BUFFER_SIZE)) {    /* No room for NULL */
 
-  if ((iErr = !(iType & iLex)))
-    ReportError (pibIn, RE_LEXEXPECTED, vrgszLexTypes[iType], szLex);
+    pbufEOBOld = pbufEOB;        /* Save EOB */
+    lFilePos = ftell (pibIn->pfileIn);    /* Save position */
+    while (*(--pbufEOB) != CH_EOLN)
+      ;                /* Move EOB to last EOLN */
+    *pbufEOB = '\0';        /* Overwrite EOLN with NULL */
 
-  return (iErr);
+    if ((lDelta = (long) (pbufEOB - pbufEOBOld)))
+      fseek (pibIn->pfileIn, lDelta, SEEK_CUR);    /* Backup file ptr */
+  } /* if */
 
-} /* ENextLex */
+  else *pbufEOB = '\0'; /* Append NULL */
+
+} /* PreventLexSplit */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
    SkipComment
 
    Skips over the comment in the input buffer, the leading delimiter
@@ -537,7 +908,7 @@ void SkipComment (PINPUTBUF pibIn)
 } /* SkipComment */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
    NextChar
 
    Returns the next character in the input buffer without advancing
@@ -557,7 +928,7 @@ char NextChar (PINPUTBUF pibIn)
 } /* NextChar */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
    GetOptPunct
 
    Advances over the optional punctuation chPunct.  Allows syntax where either
@@ -569,7 +940,8 @@ char NextChar (PINPUTBUF pibIn)
 int GetOptPunct (PINPUTBUF pibIn, PSTR szLex, char chPunct)
 {
   int iReturn, iType;
-                    /* Assigning iReturn makes optional */
+
+  /* Assigning iReturn makes optional */
   iReturn = SkipWhitespace (pibIn);
   if (NextChar (pibIn) == chPunct) {
     iReturn = TRUE;
@@ -580,7 +952,7 @@ int GetOptPunct (PINPUTBUF pibIn, PSTR szLex, char chPunct)
 } /* GetOptPunct */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
    GetPunct
 
    Tries to get the given punctuation from the input buffer.
@@ -597,7 +969,7 @@ int GetPunct (PINPUTBUF pibIn, PSTR szLex, char chPunct)
 } /* GetPunct */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
    EGetPunct
 
    Tries to get the given punctuation from the input buffer.
@@ -620,7 +992,7 @@ int EGetPunct (PINPUTBUF pibIn, PSTR szLex, char chPunct)
 } /* EGetPunct */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
    EatStatement
 
    Eats buffer to the statement terminator.
@@ -644,12 +1016,13 @@ void EatStatement (PINPUTBUF pib)
 } /* EatStatement */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
    GetStatement
 
    Gets the next statement from the input buffer.  The buffer is
-   read until a statement terminator ';' is found.  The statement is
-   unprocessed.  Syntactical validity will be checked later.
+   read until a statement terminator ';' is found.  White spaces
+   before the ';' are removed; the statement is otherwise unprocessed.
+   Syntactical validity will be checked later.
 
    The buffer szStmt is assumed to be of type PSTREQN of size MAX_EQN.
 */
@@ -669,18 +1042,24 @@ void GetStatement (PINPUTBUF pibIn, PSTR szStmt)
     c = *pibIn->pbufCur;
     while (!fDone) {
       if (*pibIn->pbufCur) {
-        if (!(fDone = (NextChar (pibIn) == CH_STMTTERM)))
-          if (i < MAX_EQN-2) {
+        if (!(fDone = (NextChar (pibIn) == CH_STMTTERM))) {
+          if (i < MAX_EQN - 2) {
             if ((szStmt[i++] = *pibIn->pbufCur++) == CH_EOLN)
               pibIn->iLineNum++;
           } /* if */
-          else
+          else {
             ReportError (pibIn, RE_EQNTOOLONG | RE_FATAL, NULL, NULL);
+          }
+        }
       } /* if pibIn->pbufCur */
       else
       fDone = (FillBuffer (pibIn) == EOF);
     } /* while */
 
+    /* remove white spaces going backward - FB 28/2/98 */
+    while (isspace(szStmt[i-1]))
+      i = i - 1;
+
     szStmt[i] = '\0';
 
   } /* if */
@@ -692,12 +1071,12 @@ void GetStatement (PINPUTBUF pibIn, PSTR szStmt)
 } /* GetStatement */
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
    NextListItem
 
    Copies the next list item to szLex if it is a lexical element
-   indicated in the bit flags bIdTypes.  If the fItemNum flag is
-   non-zero, a separator must appear in the list.
+   indicated in the bit flags bIdTypes.  If the fItemNum 
+   flag is non-zero, a separator must appear in the list.
 
    Returns positive if a valid item is found.
    Returns negative if szLex is an invalid item.
@@ -714,7 +1093,7 @@ int NextListItem (PINPUTBUF pibIn, PSTR szLex,
       || GetOptPunct (pibIn, szLex, ',')) {
     if (NextChar (pibIn) != cListTerm) {
       NextLex (pibIn, szLex, &iType);
-      if (iType & bIdTypes)
+      if ((iType & bIdTypes))
     iReturn = 1;
       else
     iReturn = -1;
@@ -726,51 +1105,121 @@ int NextListItem (PINPUTBUF pibIn, PSTR szLex,
 } /* NextListItem */
 
 
-/* -----------------------------------------------------------------------------
-   GetFuncArgs
+/* ---------------------------------------------------------------------------
+   SkipWhitespace
 
-   Gets nArgs arguments to a "function" from pibIn.  The argument
-   list must be in parentheses.
-
-   rgiArgTypes[] is a profile specifying which type each argument must be.
-   rgszArgs[] is an array of PSTRLEX buffers to hold the arguments.
-
-   Returns TRUE on success, FALSE on an error.  Errors are reported,
-   but the statement is not flushed.
+   Skips over whitespace of input buffer.  Returns non-zero if something
+   has been skipped.
 */
 
-BOOL GetFuncArgs (PINPUTBUF pibIn,
-                  int nArgs, int rgiArgTypes[], PSTR szArgs)
+int SkipWhitespace (PINPUTBUF pibIn)
 {
-  BOOL bOK = TRUE;
-  int i, iType;
-  PSTRLEX szPunct;
+  char c;
+  int fSkipped = 0;
 
-  if (!(bOK = GetPunct (pibIn, szPunct, CH_LPAREN))) {
-    szPunct[1] = CH_LPAREN;
-    ReportError (pibIn, RE_EXPECTED, szPunct, NULL);
-  } /* if */
+  if (!pibIn)
+    return 0;
 
-  for (i = 0; i < nArgs && bOK; i++, szArgs += MAX_LEX) {
-    if (i)
-      if (!(bOK = GetOptPunct (pibIn, szArgs, ','))) {
-    *(szArgs+1) = ',';
-    ReportError(pibIn, RE_EXPECTED, szArgs, NULL);
-    break;                /* Error: Stop getting args */
+  if (!*pibIn->pbufCur && pibIn->pfileIn)
+    FillBuffer (pibIn);
+
+  /* Skip Spaces, Tabs and Newlines */
+
+  while (isspace(c = *pibIn->pbufCur) || c == CH_COMMENT) {
+    fSkipped = 1;
+    if (IsComment(&c))
+      SkipComment (pibIn);
+
+    else {
+      if (c == '\n')
+    pibIn->iLineNum++;
+
+      if (!*(++pibIn->pbufCur) && pibIn->pfileIn)
+    if (FillBuffer (pibIn) == EOF)
+      break;
+    } /* else */
+  } /* while */
+
+  return (fSkipped);
+
+} /* SkipWhitespace */
+
+
+/* ---------------------------------------------------------------------------
+   UnrollEquation
+
+   Copy szEqn in szEqnU, replacing bracketed expressions evaluating in
+   <number> by _number. Expressions can be composed of integers, the 
+   4 basic arithmetic operators, parentheses and 'i' which stands for the
+   argument index passed to the routine.
+   Examples:
+   y[0] -> y_0
+   y[1 + 1] -> y_2
+   y[i * 2] -> y_4 if index = 2
+*/
+
+void UnrollEquation (PINPUTBUF pibIn, long index, PSTR szEqn, PSTR szEqnU)
+{
+  int j = 0, k = 0, m;
+  BOOL bExpress = FALSE;
+  PSTRLEX szExpression;
+
+  while ((szEqn[j] != '\0') && (k < MAX_EQN - 1)) {
+    if (bExpress) { /* bracketed expressions found: scan it up to ] included */
+      /* copy the expression to a temporary string */
+      m = 0;
+      while ((szEqn[j] != '\0') && (szEqn[j] != ']') && (m < MAX_EQN - 1)) {
+        szExpression[m] = szEqn[j]; 
+        j++;
+        m++;
       }
+      if (szEqn[j] == ']') { /* skip and exit expression parsing mode */
+        j++;
+        bExpress = FALSE;
+      }
+      if ((szEqn[j] != '\0') && (m == MAX_EQN - 1))
+        ReportError (pibIn, RE_EQNTOOLONG | RE_FATAL, NULL, 
+                     "(Occured while unrolling a loop)");
+      szExpression[m] = '\0'; /* terminate szExpression */
 
-    NextLex (pibIn, szArgs, &iType);
-    if (!(bOK &= (iType & rgiArgTypes[i]) > 0))
-      ReportError(pibIn, RE_LEXEXPECTED,
-              vrgszLexTypes[rgiArgTypes[i]], szArgs);
-  } /* for */
+      /* compute expression and put back the result in szExpression */
+      sprintf (szExpression, "%ld", 
+               EvaluateExpression (pibIn, index, szExpression));
 
-  if (!(bOK = GetPunct (pibIn, szPunct, CH_RPAREN))) {
-    szPunct[1] = CH_RPAREN;
-    ReportError (pibIn, RE_EXPECTED, szPunct, NULL);
-  } /* if */
+      /* copy szExpression into szEqnU */
+      m = 0;
+      while ((szExpression[m] != '\0') && (m < MAX_EQN - 1)) {
+        szEqnU[k] = szExpression[m]; 
+        k++;
+        m++;
+      }
+    }
+    else switch (szEqn[j]) {
+      case '[': /* replace by _ and enter expression parsing mode */
+        szEqnU[k] = '_';
+        j++;
+        k++;
+        bExpress = TRUE;
+        break;
 
-  return (bOK);
-} /* GetFuncArgs */
+      case ']': /* should have been eaten in expression parsing mode */
+        ReportError (pibIn, RE_UNEXPECTED | RE_FATAL, "]", 
+                     "(Could be nested brackets)");
 
+      default: /* copy and advance */
+        szEqnU[k] = szEqn[j]; 
+        j++;
+        k++;
+        break;
+    }
+  } /* while */
+  if ((szEqn[j] != '\0') && (k == MAX_EQN - 1))
+    ReportError (pibIn, RE_EQNTOOLONG | RE_FATAL, NULL, 
+                 "(Occured while unrolling a loop)");
+    
+  /* terminate szEqnU */
+  szEqnU[k] = '\0';
 
+} /* UnrollEquation */
+
+/* End */
