@@ -5,11 +5,25 @@
 
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
    -- Revisions -----
-     Logfile:  SCCS/s.lexerr.c
-    Revision:  1.4
-        Date:  03 Sep 1995
-     Modtime:  10:20:26
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -23,7 +37,7 @@
 #include "lexerr.h"
 
 
-/* -----------------------------------------------------------------------------
+/* ---------------------------------------------------------------------------
    ReportError
 
    Reports error iCode to terminal (one of RE_) and optional
@@ -50,15 +64,18 @@ void ReportError (PINPUTBUF pibIn, WORD wCode, PSTR szMsg, PSTR szAltMsg)
     } /* else */
   } /* if */
 
-  if (pibIn)
-    if (pibIn->pfileIn || pibIn->iLNPrev) /* Line number is valid */
+  if (pibIn) {
+    if (pibIn->pfileIn || pibIn->iLNPrev) { /* Line number is valid */
       printf ("line %d: ", pibIn->iLineNum);
-
-    else if (wCode != RE_FILENOTFOUND) { /* Dummy pibIn, show buffer */
-      PSTRLEX szTmp;
-      szTmp[MAX_LEX-1] = '\0';
-      printf ("'%s'...\n  ", strncpy (szTmp, pibIn->pbufOrg, MAX_LEX-1));
-    } /* if */
+    }
+    else {
+      if (wCode != RE_FILENOTFOUND) { /* Dummy pibIn, show buffer */
+        PSTRLEX szTmp;
+        szTmp[MAX_LEX-1] = '\0';
+        printf ("'%s'...\n  ", strncpy (szTmp, pibIn->pbufOrg, MAX_LEX-1));
+      } /* if */
+    }
+  }
 
   switch (wCode) {
 
@@ -139,42 +156,17 @@ void ReportError (PINPUTBUF pibIn, WORD wCode, PSTR szMsg, PSTR szAltMsg)
     break;
 
   case RE_TOOMANYVARS:
-    printf ("Too many %s declarations. Limit is %d.\n",
-             szMsg, *(PINT)szAltMsg);
+    printf ("Too many %s declarations. Limit is %d.\n", szMsg, *(PINT)szAltMsg);
     break;
 
-  /* Simulation input errors */
-
-  case RE_ERRORSINEXP:
-    printf ("Bad definition of experiment %d\n", *(PINT)szMsg);
+  case RE_POSITIVE:
+    printf ("Positive number expected.");
     break;
 
-  case RE_NOOUTPUTS:
-    printf ("Experiment %d has no outputs specified\n", *(PINT)szMsg);
+  case RE_NAMETOOLONG:
+    printf ("Name %s exceed %d characters.", szMsg, MAX_NAME);
     break;
 
-  case RE_NOOUTTIMES:
-    printf ("Experiment %d has no output times specified\n", *(PINT)szMsg);
-    break;
-
-  case RE_INTEGERR:
-    printf ("DifSub() returned error %ld: %s\n", *(PLONG) szMsg, szAltMsg);
-    break;
-
-  case RE_SPECERR:
-    printf ("in specification: %s\n", szMsg);
-    break;
-
-  case RE_INSUF_POINTS:
-    printf ("Insufficient points in file \"%s\"\n", szMsg);
-    break;
-
-  case RE_DEFOTHER:
-    printf ("Already defining other experiment %s\n", szMsg);
-    break;
-
-  case RE_MAXMIN_RANGE:
-    printf ("Max is less than min\n");
   } /* switch */
 
   printf ("\n");
