@@ -5,6 +5,20 @@
    
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
    -- Revisions -----
      Logfile:  %F%
     Revision:  %I%
@@ -76,16 +90,19 @@ typedef struct tagIFN {		/* Input Function struct */
 } IFN, *PIFN;	/* struct tagIFN */
 
 
-
 /* ----- Macros  */
 
 /* ----- Globals/Externals  */
 
 /* ----- Prototypes  */
 
-int GetFnType (PSTR szName);
+int  GetFnType (PSTR szName);
+void InitIFN (PIFN pifn);
+BOOL DefDepParm (PSTR szLex, PDOUBLE pdValue, HANDLE *phvar);
+BOOL GetInputArgs (PINPUTBUF pibIn, PIFN pifn);
+BOOL GetNNumbers (PINPUTBUF pibIn, PSTR szLex, int nNumbers, PDOUBLE rgd);
+BOOL GetNDoses (PINPUTBUF pibIn, PSTR szLex, PIFN pifn);
 BOOL GetInputFn (PINPUTBUF pibIn, PSTR sz, PIFN pifn);
 
-
 #define _LEXFN_INCLUDED
 #endif
