@@ -3,7 +3,21 @@
    written by Don Maszle
    15 September 1991
    
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1993-2004.  Don Maszle, Frederic Bois.  All rights reserved.
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
    -- Revisions -----
      Logfile:  %F%
@@ -40,19 +54,13 @@
                         values specified in Print().
 */
 
-#ifdef __LOCAL_HDR__
-#include "stdlib.h"
-#include "stdio.h"
-#include "string.h"
-
-#else
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#endif
 
 #include "mod.h"
-
+#include "modi.h"
+#include "modo.h"
 
 /***********************************************************************
    main -- Entry point for the PBPK simulation model preprocessor
@@ -63,8 +71,13 @@ int main (int nArg, PSTR rgszArg[])
 {
   INPUTINFO info;
 
-  fprintf (stderr, "\nModel Generator Program " VSZ_VERSION "\n");
-  fprintf (stderr, VSZ_COPYRIGHT "\n\n");
+  printf ("\n________________________________________\n");
+  printf ("\nMod - Model Generator Program " VSZ_VERSION "\n\n");
+  printf (VSZ_COPYRIGHT "\n\n");
+
+  printf ("MCSim comes with ABSOLUTELY NO WARRANTY;\n"
+          "This is free software, and you are welcome to redistribute it\n"
+          "under certain conditions; see the GNU General Public License.\n\n");
 
 #ifdef _MACOS_
   /* for the Macintosh there are no command line parameters 
