@@ -3,7 +3,21 @@
    written by Don Maszle
    26 September 1991
    
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1993-2004.  Don Maszle, Frederic Bois.  All rights reserved.
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
    -- Revisions -----
      Logfile:  %F%
@@ -12,6 +26,7 @@
      Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
+
 */
 
 #ifndef _MOD_INCLUDED
@@ -24,8 +39,8 @@
 /* ----- Constants  */
 
 /* Version and copyright */
-#define VSZ_VERSION "v4.0"
-#define VSZ_COPYRIGHT "Copyright (c) 1993-96.  Don Maszle, Frederic Bois.  All rights reserved."
+#define VSZ_VERSION "v5.0.0"
+#define VSZ_COPYRIGHT "Copyright (c) 1993-2004 by F. Bois & D. Maszle.  All rights reserved."
 
 /* The time variable accessible to the user */
 #define VSZ_TIME "t"
@@ -69,7 +84,7 @@
 */
 
 #define ID_TYPEMASK     0xF000 /* Allow up to 15 variable types */
-#define ID_SPACEFLAG    0x0800 /* To flag for formatting eqns */
+#define ID_SPACEFLAG    0x0F00 /* To flag for formatting eqns */
 #define ID_INDEXMASK    0x07FF /* Index for symbol table */
 #define MAX_VARS        (ID_SPACEFLAG) /* Max number of vars of a type */
 
@@ -127,19 +142,5 @@ typedef struct tagINPUTINFO {
      : (kmCode) == KM_DYNAMICS ? CN_DYNAMICS \
      : 0)
 
-
-/* ----- Globals/Externals */
-
-
-/* ----- Prototypes */
-
-void DeclareModelVar (PINPUTBUF, PSTR, int);
-void DefineVariable (PINPUTBUF, PSTR, PSTR, int);
-int  GetKeywordCode (PSTR, PINT);
-int  GetVarType (PVMMAPSTRCT, PSTR);
-PVMMAPSTRCT  GetVarPTR (PVMMAPSTRCT, PSTR);
-void ReadModel (PINPUTINFO, PSTR);
-void WriteModel (PINPUTINFO, int, PSTR *);
-
 #define _MOD_INCLUDED
 #endif
