@@ -5,6 +5,20 @@
    
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
    -- Revisions -----
      Logfile:  %F%
     Revision:  %I%
@@ -27,31 +41,36 @@
 #include "lexerr.h"
 #include "mod.h"
 #include "lexfn.h"
+#include "modd.h"
+#include "modi.h"
 
-/* List of valid C math functions that can be used in equations --
-   
-   These are all ANSI standard functions, and return the type DOUBLE
+/* List of valid functions that can be used in equations,
+   they all return the type DOUBLE.
 */
 
 PSTR vrgszMathFuncs[] = {
-  "acos", "asin", "atan", "atan2", "ceil",
-  "cos",  "cosh", "exp",  "fabs",  "floor",
-  "fmod", "log",  "log10","pow",   "sin",
-  "sinh", "sqrt", "tan",  "tanh",
-  "CDFNormal", "erfc", "lnDFNormal", "lnGamma", "UniformRandom",
-  "NormalRandom",
-  "" /* End flag */
-}; /* vrgszMathFuncs[] = */
+  /* standard math functions */
+  "acos", "asin", "atan", "atan2", "ceil", "cos", "cosh", "exp", "fabs", 
+  "floor", "fmod", "log", "log10", "pow", "sin", "sinh", "sqrt", "tan", "tanh",
+
+  /* special functions defined in random.c */
+  "CDFNormal", "erfc", "lnDFNormal", "lnGamma", 
+
+  /* random sampling related routines defined in random.c */
+  "BinomialRandom", "BetaRandom", "BinomialBetaRandom", "Chi2Random", 
+  "ExpRandom", "InvGGammaRandom", "GammaRandom", "GetSeed", "GGammaRandom",
+  "LogNormalRandom", "LogUniformRandom", "NormalRandom", "PiecewiseRandom",
+  "PoissonRandom", "SetSeed", "TruncLogNormalRandom", "TruncNormalRandom", 
+  "UniformRandom",
+
+  /* End flag */
+  ""
+};
 
 /* Global used by modd.c and modo.c as a flag */
 char vszHasInitializer[] = "0.0; /* Redefined later */";
       
 
-/* Prototypes for this file */
-
-PSTR CopyString (PSTR);
-
-
 /* Functions */
 
 /* ----------------------------------------------------------------------------
@@ -530,6 +549,33 @@ void DefineVariable (PINPUTBUF pibIn, PSTR szName, PSTR szEqn, int iKWCode)
   pvm = GetVarPTR (pinfo->pvmGloVars, szName);
   hGloVarType = TYPE(pvm);
 
+  /* the variable has been found but was defined as a local for
+     another section that the current one, it need to be redefined
+     as local, so we undo the above assignments */
+  if (((pinfo->wContext == CN_DYNAMICS) && 
+       ((hGloVarType == ID_LOCALSCALE) ||
+        (hGloVarType == ID_LOCALJACOB) || 
+        (hGloVarType == ID_LOCALCALCOUT))) || 
+
+      ((pinfo->wContext == CN_SCALE)    && 
+       ((hGloVarType == ID_LOCALDYN)   ||
+        (hGloVarType == ID_LOCALJACOB) || 
+        (hGloVarType == ID_LOCALCALCOUT))) ||
+
+      ((pinfo->wContext == CN_JACOB)    && 
+       ((hGloVarType == ID_LOCALDYN)   ||
+        (hGloVarType == ID_LOCALSCALE) || 
+        (hGloVarType == ID_LOCALCALCOUT))) ||
+
+      ((pinfo->wContext == CN_CALCOUTPUTS) && 
+       ((hGloVarType == ID_LOCALDYN)   ||
+        (hGloVarType == ID_LOCALSCALE) || 
+        (hGloVarType == ID_LOCALJACOB)))) {
+
+    pvm = NULL;
+    hGloVarType = TYPE(pvm);
+  }
+
   if ((hGloVarType != ID_INPUT || pinfo->wContext != CN_GLOBAL)
       && !VerifyEqn (pibIn, szEqn))
     return; /* Errors reported in Verify eqn */
@@ -559,12 +605,14 @@ void DefineVariable (PINPUTBUF pibIn, PSTR szName, PSTR szEqn, int iKWCode)
       DefineScaleEqn (pibIn, szName, szEqn, hGloVarType);
       break;
 
-    case CN_CALCOUTPUTS:
-      if (hGloVarType == ID_OUTPUT)
+    case CN_CALCOUTPUTS: /* FB added ID_NULL and RE_FATAL, 1 mar 98 */
+      /* only outputs and local variables can be assigned in CalcOutputs */
+      if ((hGloVarType == ID_OUTPUT) || (hGloVarType == ID_NULL))
         DefineCalcOutEqn (pibIn, szName, szEqn, hGloVarType);
       else
-        ReportError (pibIn, RE_BADCONTEXT, szName,
-                     "Only outputs can be defined in CalcOutputs{} section.");
+        ReportError (pibIn, RE_BADCONTEXT  | RE_FATAL, szName,
+                     "Only outputs and local variables can be "
+                     "defined in CalcOutputs{} section.");
       break;
 
     default:
@@ -584,7 +632,6 @@ void DefineVariable (PINPUTBUF pibIn, PSTR szName, PSTR szEqn, int iKWCode)
    description file, it should be assured that this procedure is only
    called in that context.
 
-   If the declaration is duplicate 
    The number of States, Outputs, or Inputs is incremented, and 
    nominal value and defining equations strings are initialized to NULL.
 */
