@@ -6,6 +6,20 @@
    
    Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
 
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
    -- Revisions -----
      Logfile:  %F%
     Revision:  %I%
@@ -15,6 +29,9 @@
    -- SCCS  ---------
 
    Output the model.c file.
+
+   Modified on 07/07/97 by FB - added yourcode.h to WriteIncludes().
+ 
 */
 
 #include <stdio.h>
@@ -27,19 +44,12 @@
 #include "mod.h"
 #include "lexfn.h"
 #include "lexerr.h"
+#include "modi.h"
+#include "modd.h"
+#include "modo.h"
 
 
-/* Extra Constants, typedefs, prototypes */
-
-#define ALL_VARS (0)
-
-#ifndef FILENAME_MAX
-#define FILENAME_MAX 40
-#endif
-
-#define WriteIndexName(pfile, pvm)  (fprintf ((pfile), "ID_%s", (pvm)->szName))
-
-typedef int (*PFI_CALLBACK) (PFILE, PVMMAPSTRCT, PVOID);
+/* Global Variables */
 
 static char *vszModelFilename = NULL;
 static char *vszModGenName = NULL;
@@ -60,8 +70,6 @@ char *vszIFNTypes[] = { /* Must match defines in lexfn.h */
 }; /* vszIFNTypes[] = */
 
 
-/* Global Variables */
-
 int vnStates, vnOutputs, vnInputs, vnParms, vnModelVars;
 
 /* ----------------------------------------------------------------------------
@@ -78,12 +86,12 @@ int ForAllVar (PFILE pfile, PVMMAPSTRCT pvm, PFI_CALLBACK pfiFunc,
   
   while (pvm) {
     if (hType == ALL_VARS          /* Do for all ...*/
-        || TYPE(pvm) == hType)     /* ..or do only for vars of hType */
+        || TYPE(pvm) == hType) {   /* ..or do only for vars of hType */
       if (pfiFunc)
         iTotal += (*pfiFunc) (pfile, pvm, pinfo);
       else
         iTotal++; /* No func! just count */
-
+    }
     pvm = pvm->pvmNextVar;
   } /* while */
 
@@ -154,7 +162,7 @@ void WriteHeader (PFILE pfile, PSTR szName, PVMMAPSTRCT pvmGlo)
   fprintf(pfile, "   ___________________________________________________\n\n");
   fprintf (pfile, "   Model File:  %s\n\n", vszModelFilename);
   fprintf (pfile, "   Date:  %s\n", ctime(&ttTime));
-  fprintf (pfile, "   Created by:  \"%s-%s\"\n", vszModGenName, VSZ_VERSION);
+  fprintf (pfile, "   Created by:  \"%s %s\"\n", vszModGenName, VSZ_VERSION);
   fprintf (pfile, "    -- a model preprocessor by Don Maszle\n");
   fprintf(pfile, "   ___________________________________________________\n\n");
 
@@ -191,6 +199,7 @@ void WriteIncludes (PFILE pfile)
   
   fprintf (pfile, "#include \"modelu.h\"\n");
   fprintf (pfile, "#include \"random.h\"\n");
+  fprintf (pfile, "#include \"yourcode.h\"\n");
 
 } /* WriteIncludes */
 
@@ -206,7 +215,8 @@ int WriteOneDecl (PFILE pfile, PVMMAPSTRCT pvm, PVOID pInfo)
   assert(TYPE(pvm) != ID_OUTPUT);
   assert(TYPE(pvm) != ID_STATE);
 
-  if (TYPE(pvm) & (ID_LOCALDYN | ID_LOCALSCALE | ID_LOCALJACOB))
+  if (TYPE(pvm) & (ID_LOCALDYN | ID_LOCALSCALE | ID_LOCALJACOB | 
+                   ID_LOCALCALCOUT))
     fprintf (pfile, "  ");
 
   fprintf (pfile, "double %s;\n", pvm->szName);
@@ -255,7 +265,7 @@ void WriteDecls (PFILE pfile, PVMMAPSTRCT pvmGlo)
   fprintf (pfile, "\n  /* For export.  Keep track of who we are. */\n");
   fprintf (pfile, "char szModelDescFilename[] = \"%s\";\n", vszModelFilename);
   fprintf (pfile, "char szModelSourceFilename[] = __FILE__;\n");
-  fprintf (pfile, "char szModelGenAndVersion[] = \"%s-%s\";\n",
+  fprintf (pfile, "char szModelGenAndVersion[] = \"%s %s\";\n",
     vszModGenName, VSZ_VERSION);
 
   fprintf (pfile, "\n  /* Externs */\n");
@@ -447,7 +457,7 @@ void WriteInitModel (PFILE pfile, PVMMAPSTRCT pvmGlo)
   fprintf (pfile, "   Should be called to initialize model variables at\n");
   fprintf (pfile, "   the beginning of experiment before reading\n");
   fprintf (pfile, "   variants from the simulation spec file.\n*/\n\n");
-  fprintf (pfile, "void InitModel()\n{\n");
+  fprintf (pfile, "void InitModel(void)\n{\n");
   fprintf (pfile, "/*-- Initialize things in the order that they appear in\n"
       "     model definition file so that dependencies are\n"
       "     handled correctly. */\n\n");
@@ -494,21 +504,23 @@ void WriteInitExpt (PFILE pfile, PVMMAPSTRCT pvmGlo)
 void TranslateID (PINPUTBUF pibDum, PFILE pfile, PSTR szLex, int iEqType)
 {
   int iKWCode, fContext;
-    
+  long iLowerB, iUpperB;
+   
   iKWCode = GetKeywordCode (szLex, &fContext);
+
   switch (iKWCode) {
 
   case KM_DXDT: {
     int iArg = LX_IDENTIFIER;
     PVMMAPSTRCT pvm = NULL;
     
-    if (GetFuncArgs (pibDum, 1, &iArg, szLex)
+    if (GetFuncArgs (pibDum, 1, &iArg, szLex, &iLowerB, &iUpperB)
         && (pvm = GetVarPTR(vpvmGloVarList, szLex))
         && TYPE(pvm) == ID_STATE)
       fprintf (pfile, "%s", GetName (pvm, NULL, "rgDerivs", ID_DERIV));
     else
       ReportError (pibDum, RE_BADSTATE | RE_FATAL, (pvm? szLex : NULL), NULL);
-    } /* KM_DXDT: */
+    } /* KM_DXDT block */
     break;
 
   case KM_NULL: {
@@ -526,13 +538,14 @@ void TranslateID (PINPUTBUF pibDum, PFILE pfile, PSTR szLex, int iEqType)
     } /* if */
  
     else {
-      fprintf(pfile, "%s", GetName(pvm, "rgModelVars", NULL, ID_NULL));
-      if (TYPE(pvm) == ID_INPUT) {
-        if (iEqType == KM_SCALE)
-          fprintf(pfile, ".dMag"); /* Can only scale with magnitude */
-        else
-          fprintf(pfile, ".dVal"); /* Otherwise use current value */
-      } /* if */
+      if (iEqType == KM_SCALE)  
+        /* fixed by FB - 1 mar 98. This prints a "vrgModelVars" */
+        fprintf(pfile, "%s", GetName(pvm, NULL, NULL, ID_NULL));
+      else
+        fprintf(pfile, "%s", GetName(pvm, "rgModelVars", NULL, ID_NULL));
+
+      if (TYPE(pvm) == ID_INPUT)
+        fprintf(pfile, ".dVal");  /* Use current value */
     } /* else */
 
     } /* KM_NULL: */
@@ -576,7 +589,7 @@ void TranslateEquation (PFILE pfile, PSTR szEqn, int iEqType)
   } /* if */
 
   do {
-    if (iType == LX_IDENTIFIER)   /* Process Identifier */
+    if (iType == LX_IDENTIFIER)  /* Process Identifier */
       TranslateID (pibDum, pfile, szLex, iEqType);
     
     else if ((iType == LX_EQNPUNCT || iType == LX_PUNCT) && IsComment(szLex)) {
@@ -625,8 +638,7 @@ int WriteOneEquation (PFILE pfile, PVMMAPSTRCT pvm, PVOID pInfo)
         exit(0);
       }
 
-      fprintf (pfile, "  %s%s = ", GetName (pvm, NULL, NULL, ID_NULL),
-        (TYPE(pvm) == ID_INPUT ? ".dMag" : "") );
+      fprintf (pfile, "  %s = ", GetName (pvm, NULL, NULL, ID_NULL));
       break;
 
     case KM_CALCOUTPUTS:
@@ -657,20 +669,18 @@ void WriteCalcDeriv (PFILE pfile, PVMMAPSTRCT pvmGlo, PVMMAPSTRCT pvmDyn)
     printf ("No Dynamics{} equations.\n");
 
   fprintf (pfile, "/* Dynamics section */\n");
-  fprintf (pfile, "void CalcDeriv (double  rgModelVars[],\n");
-  fprintf (pfile, "   double  rgDerivs[],\n");
-  fprintf (pfile, "   PDOUBLE pdTime)\n");
-  fprintf (pfile, "{\n");
+  fprintf (pfile, "void CalcDeriv (double  rgModelVars[], ");
+  fprintf (pfile, "double  rgDerivs[], PDOUBLE pdTime)\n{\n");
   fprintf (pfile, "  /* Local variables */\n");
 
   ForAllVar (pfile, pvmGlo, &WriteOneDecl, ID_LOCALDYN, NULL);
 
   fprintf (pfile, "\n");
-  fprintf (pfile, "  CalcInputs (pdTime);    /* Get new input vals */\n");
+  fprintf (pfile, "  CalcInputs (pdTime);  /* Get new input vals */\n");
 
   ForAllVar (pfile, pvmDyn, &WriteOneEquation, ALL_VARS, (PVOID) KM_DYNAMICS); 
  
-  fprintf (pfile, "} /* CalcDeriv */\n");
+  fprintf (pfile, "\n} /* CalcDeriv */\n\n\n");
 
 } /* WriteCalcDeriv */
 
@@ -685,17 +695,12 @@ void WriteScale (PFILE pfile, PVMMAPSTRCT pvmGlo, PVMMAPSTRCT pvmScale)
     printf ("No Scale{} equations. Null function defined.\n");
     
   fprintf (pfile, "/* Model scaling */\n");
-  fprintf (pfile, "\nvoid ScaleModel (void)\n");
+  fprintf (pfile, "void ScaleModel (void)\n");
   fprintf (pfile, "{\n");
   fprintf (pfile, "  /* Local variables */\n");
-
   ForAllVar (pfile, pvmGlo, &WriteOneDecl, ID_LOCALSCALE, NULL);
-
-  fprintf (pfile, "\n");
-
   ForAllVar (pfile, pvmScale, &WriteOneEquation, ALL_VARS, (PVOID) KM_SCALE);  
-
-  fprintf (pfile, "} /* ScaleModel */\n");
+  fprintf (pfile, "\n} /* ScaleModel */\n\n\n");
 
 } /* WriteScale */
 
@@ -705,15 +710,14 @@ void WriteScale (PFILE pfile, PVMMAPSTRCT pvmGlo, PVMMAPSTRCT pvmScale)
 */
 void WriteCalcJacob (PFILE pfile, PVMMAPSTRCT pvmGlo, PVMMAPSTRCT pvmJacob)
 {
-  fprintf (pfile, "/* Jacobian calculation */\n\n");
-  fprintf (pfile, "\nvoid CalcJacob (double rgModelVars[], double *rgdSave[]");
+  fprintf (pfile, "/* Jacobian calculation */\n");
+  fprintf (pfile, "void CalcJacob (double rgModelVars[], double *rgdSave[]");
   fprintf (pfile, ", PDOUBLE pdTime)\n");
   fprintf (pfile, "{\n");
   fprintf (pfile, "  /* Local variables */\n");
   ForAllVar (pfile, pvmGlo, &WriteOneDecl, ID_LOCALJACOB, NULL);
-  fprintf (pfile, "\n");
   ForAllVar (pfile, pvmJacob, &WriteOneEquation, ALL_VARS, (PVOID) KM_JACOB);  
-  fprintf (pfile, "} /* CalcJacob */\n");
+  fprintf (pfile, "\n} /* CalcJacob */\n\n\n");
 } /* WriteCalcJacob */
 
 
@@ -726,20 +730,13 @@ void WriteCalcOutputs (PFILE pfile, PVMMAPSTRCT pvmGlo, PVMMAPSTRCT pvmCalcOut)
     printf ("No CalcOutput{} equations. Null function defined.\n");
     
   fprintf (pfile, "/* Outputs calculation */\n");
-  fprintf (pfile, "\nvoid CalcOutputs (double  rgModelVars[],\n");
-  fprintf (pfile, "      double  rgDerivs[],\n");
-  fprintf (pfile, "      PDOUBLE pdTime)\n");
-  fprintf (pfile, "{\n");
+  fprintf (pfile, "void CalcOutputs (double  rgModelVars[], ");
+  fprintf (pfile, "double  rgDerivs[], PDOUBLE pdTime)\n{\n");
   fprintf (pfile, "  /* Local variables */\n");
-
   ForAllVar (pfile, pvmGlo, &WriteOneDecl, ID_LOCALCALCOUT, NULL);
-
-  fprintf (pfile, "\n");
-
   ForAllVar (pfile, pvmCalcOut, &WriteOneEquation, ALL_VARS, 
              (PVOID) KM_CALCOUTPUTS); 
- 
-  fprintf (pfile, "} /* CalcOutputs */\n");
+  fprintf (pfile, "\n}  /* CalcOutputs */\n\n\n");
 
 } /* WriteCalcOutputs */
 
@@ -924,11 +921,11 @@ int AssertExistsEqn (PFILE pfile, PVMMAPSTRCT pvm, PVOID pInfo)
 
   if (pvm->szEqn != vszHasInitializer) { /* Don't count these! */
     if (pvmDyn) {
-      if (!(iReturn = (int) GetVarPTR (pvmDyn, pvm->szName)))
+      if (!(iReturn = (GetVarPTR (pvmDyn, pvm->szName) != NULL)))
         ReportError (NULL, RE_NODYNEQN, pvm->szName, NULL);
     }
     else
-      if (!(iReturn = (int) pvm->szEqn))
+      if (!(iReturn = (pvm->szEqn != NULL)))
         ReportError (NULL, RE_NOINPDEF, pvm->szName, NULL);
   } /* if */
 
@@ -1008,10 +1005,12 @@ void WriteModel (PINPUTINFO pinfo, int nArg, PSTR rgszArgs[])
   MakeCodeFilename (szFileWithExt, 
                     (nArg > 2 ? rgszArgs[2] : vszFilenameDefault));
 
-  if ((pfile = fopen (szFileWithExt, "w"))) {
+  pfile = fopen (szFileWithExt, "w");
+  if (pfile) {
 
     /* Keep track of the model description file and generator name */
     vszModelFilename = pinfo->szInputFilename;
+
 #ifdef _MACOS_
     vszModGenName = "Mod";
 #else
@@ -1032,11 +1031,11 @@ void WriteModel (PINPUTINFO pinfo, int nArg, PSTR rgszArgs[])
     WriteCalcOutputs (pfile, pinfo->pvmGloVars, pinfo->pvmCalcOutEqns);
 
     fclose (pfile);
+
     printf ("\n* Created model file '%s'\n", szFileWithExt);
 
   } /* if */
-
-  if (!pfile)
+  else
     ReportError (NULL, RE_CANNOTOPEN | RE_FATAL,
                  szFileWithExt, "...in WriteModel ()");
 
