@@ -17,10 +17,10 @@
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
    -- Revisions -----
-     Logfile:  SCCS/s.getopt.c
-    Revision:  1.2
-        Date:  28 Jul 1997
-     Modtime:  03:16:25
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
