@@ -3,7 +3,7 @@
    written by Don Maszle
    15 September 1991
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1991-2004.  Don Maszle, Frederic Bois.  All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -19,19 +19,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-   contact
-     Frdric Bois / Don Maszle
-     BEHS, School of Public Health
-     University of California at Berkeley
-     Berkeley, CA 94720
-
-     fbois@diana.lbl.gov
-
    -- Revisions -----
-     Logfile:  SCCS/s.lexerr.c
-    Revision:  1.18
-        Date:  14 Nov 1997
-     Modtime:  06:38:56
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -39,11 +31,6 @@
 
 */
 
-#ifdef _MACOSLEVEL2_
-#include <PPC Macheaders>
-#include "mac.h"
-#endif
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -80,15 +67,18 @@ void ReportError (PINPUTBUF pibIn, WORD wCode, PSTR szMsg, PSTR szAltMsg)
     } /* else */
   } /* if */
 
-  if (pibIn)
-    if (pibIn->pfileIn || pibIn->iLNPrev) /* Line number is valid */
+  if (pibIn) {
+    if (pibIn->pfileIn || pibIn->iLNPrev) { /* Line number is valid */
       printf ("line %d: ", pibIn->iLineNum);
-
-    else if (wCode != RE_FILENOTFOUND) { /* Dummy pibIn, show buffer */
-      PSTRLEX szTmp;
-      szTmp[MAX_LEX-1] = '\0';
-      printf ("'%s'...\n  ", strncpy (szTmp, pibIn->pbufOrg, MAX_LEX-1));
-    } /* if */
+    }
+    else {
+      if (wCode != RE_FILENOTFOUND) { /* Dummy pibIn, show buffer */
+        PSTRLEX szTmp;
+        szTmp[MAX_LEX-1] = '\0';
+        printf ("'%s'...\n  ", strncpy (szTmp, pibIn->pbufOrg, MAX_LEX-1));
+      } /* if */
+    }
+  }
 
   switch (wCode) {
 
@@ -145,11 +135,11 @@ void ReportError (PINPUTBUF pibIn, WORD wCode, PSTR szMsg, PSTR szAltMsg)
     break;
 
   case RE_LEVINEXPT:
-    printf("Level statement enclosed in Experiment statement");
+    printf("Level statement enclosed in Simulation (Experiment) statement");
     break;
 
   case RE_BADDEPTH:
-    printf("Experiment statement is not at lowest level");
+    printf("Simulation (Experiment) statement is not at lowest level");
     break;
 
   case RE_BADOUTPUTVAR:
@@ -216,11 +206,11 @@ void ReportError (PINPUTBUF pibIn, WORD wCode, PSTR szMsg, PSTR szAltMsg)
     break;
 
   case RE_NOOUTPUTS:
-    printf ("Experiment %d has no outputs specified\n", *(PINT)szMsg);
+    printf ("Simulation (Experiment) %d has no outputs specified\n", *(PINT)szMsg);
     break;
 
   case RE_NOOUTTIMES:
-    printf ("Experiment %d has no output times specified\n", *(PINT)szMsg);
+    printf ("Simulation (Experiment) %d has no output times specified\n", *(PINT)szMsg);
     break;
 
   case RE_INTEGERR:
@@ -303,7 +293,7 @@ void ReportRunTimeError (PANALYSIS panal, WORD wCode, ...)
 
   case RE_OUTOFMEM:
     szMsg1 = va_arg(ap, PSTR);
-    printf ("Out of memory in %s().", szMsg1);
+    printf ("Out of memory in %s().", szMsg1); 
     break;
 
   case RE_CANNOTOPEN:
@@ -355,36 +345,61 @@ void ReportRunTimeError (PANALYSIS panal, WORD wCode, ...)
     szMsg4 = va_arg(ap, PSTR);
     szMsg5 = va_arg(ap, PSTR);
     printf ("Sampling failure for variable ");
-    printf("%s, value %s, std dev %s, min %s, max %s",
+    printf("%s, value %s, std dev %s, min %s, max %s.\n",
            szMsg1, szMsg2, szMsg3, szMsg4, szMsg5);
     break;
 
   case RE_BADMODEL:
-    printf ("Bad value in output; model is not computable\n");
+    printf ("Bad value in output; model is not computable.\n");
+    break;
+
+  case RE_NOLIKELIHOOD:
+    szMsg1 = va_arg(ap, PSTR);
+    printf ("No Distrib statement defined for the Data of variable %s.\n",
+            szMsg1);
     break;
 
   case RE_NODATASTATEMENT:
     printf ("Found model variable in 'Distrib' statement without\n");
-    printf ("  corresponding 'Data' statement\n");
+    printf ("  corresponding 'Data' statement.\n");
     break;
 
   case RE_NOPRINTSTATEMENT:
-    printf ("Found model variable in 'Distrib' statement without\n");
-    printf ("  corresponding 'Print' statement\n");
+    szMsg1 = va_arg(ap, PSTR);
+    printf ("Variable %s in 'Distrib' statement has no\n", szMsg1);
+    printf ("  corresponding 'Print' statement.\n");
+    break;
+
+  case RE_NOPDSTATEMENT:
+    szMsg1 = va_arg(ap, PSTR);
+    printf ("Variable %s in 'Distrib' statement has no\n", szMsg1);
+    printf ("  corresponding 'Print' or 'Data' statement.\n");
     break;
 
   case RE_UNEQUALNUMTIMES:
-    printf ("Found model variables in 'Distrib' statement with different\n");
-    printf ("  numbers of times in 'Print' statements\n");
+    szMsg1 = va_arg(ap, PSTR);
+    szMsg2 = va_arg(ap, PSTR);
+    szMsg3 = va_arg(ap, PSTR);
+    printf ("Variables %s and %s in 'Distrib' statement for %s\n", szMsg1, 
+            szMsg2, szMsg3);
+    printf ("  should have the same numbers of times in 'Print' statements.\n");
     break;
 
   case RE_UNEQUALTIMES:
-    printf ("Found model variables in 'Distrib' statement with different\n");
-    printf ("  values of times in 'Print' statements\n");
+    szMsg1 = va_arg(ap, PSTR);
+    szMsg2 = va_arg(ap, PSTR);
+    szMsg3 = va_arg(ap, PSTR);
+    printf ("Variables %s and %s in 'Distrib' statement for %s\n", szMsg1, 
+            szMsg2, szMsg3);
+    printf ("  must have the same times values in their 'Print' statements.\n");
     break;
 
-  case RE_DUPVARINPRINT:
-    printf ("Variable appears in two or more 'Print' statements\n");
+  case RE_DUPVARINEXPRT:
+    szMsg1 = va_arg(ap, PSTR);
+    szMsg2 = va_arg(ap, PSTR);
+    printf ("Variable %s appears in two or more '%s' statements.\n", 
+            szMsg1, szMsg2);
+    break;
 
   } /* switch */
 
