@@ -3,7 +3,7 @@
    written by Don Maszle
    15 October 1991
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1991-2004.  Don Maszle, Frederic Bois.  All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -19,19 +19,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-   contact
-     Frdric Bois / Don Maszle
-     BEHS, School of Public Health
-     University of California at Berkeley
-     Berkeley, CA 94720
-
-     fbois@diana.lbl.gov
-
    -- Revisions -----
-     Logfile:  SCCS/s.lexfn.c
-    Revision:  1.17
-        Date:  14 Nov 1997
-     Modtime:  06:38:57
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -110,7 +102,6 @@ typedef struct tagINPUTFUNCTIONMAP {
 IFM vrgifmMap[] = {    /* Input function map */
 
   {"PerDose",   IFN_PERDOSE},    /* Predef'd Input functions */
-  {"PerRate",   IFN_PERRATE},
   {"PerExp",    IFN_PEREXP},
   {"NDoses",    IFN_NDOSES},
   {"Spikes",    IFN_SPIKES},
@@ -571,7 +562,6 @@ BOOL GetInputFn (PINPUTBUF pibIn, PSTR sz, PIFN pifn)
         break;
 
       case IFN_PERDOSE:
-      case IFN_PERRATE:
       case IFN_PEREXP:
         bReturn = GetInputArgs (pibDum, pifn);
         break;
