@@ -4,7 +4,7 @@ lsodes.h
    written by Zeng G. Yuan
    19 January 1993
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1993-2004.  Don Maszle, Frederic Bois.  All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -20,19 +20,11 @@ lsodes.h
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-   contact
-     Frdric Bois / Don Maszle
-     BEHS, School of Public Health
-     University of California at Berkeley
-     Berkeley, CA 94720
-
-     fbois@diana.lbl.gov
-
    -- Revisions -----
-     Logfile:  SCCS/s.lsodes.h
-    Revision:  1.15
-        Date:  14 Nov 1997
-     Modtime:  06:39:14
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -58,7 +50,7 @@ This is the header file for the two parts
 
 
 /* ----------------------------------------------------------------------------
-   macros */
+   Macros */
 
 #define mymin(a,b) ((a) <= (b) ? (a) : (b))
 #define mymax(a,b) ((a) >= (b) ? (a) : (b))
@@ -67,11 +59,11 @@ This is the header file for the two parts
 
 
 /* ----------------------------------------------------------------------------
-   type defs */
+   Type defs */
 
 
 /* ----------------------------------------------------------------------------
-   prototypes */
+   Prototypes */
 
 int lsodes_(long *, double *, double *, double *, long *,
             double *, double *, long *, long *, long *,
@@ -79,7 +71,7 @@ int lsodes_(long *, double *, double *, double *, long *,
 
 
 /* ----------------------------------------------------------------------------
-   prototypes private to lsodes */
+   Prototypes private to lsodes */
 
 int adjlr_(long *n, long *isp, long *ldif);
 
@@ -170,8 +162,6 @@ int stode_(long *, double *, double *, long *, double *, double *,
 
 double vnorm_(long *, double *, double *);
 
-int xerrwv (char *, long, long, long,
-            long, long, long, long,
-            double, double, long);
+int xerrwv (char *, long, long, long, long, long, double, double);
 
 /* end */
