@@ -3,7 +3,7 @@
    written by Don Maszle
    7 October 1991
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1991-2004.  Don Maszle, Frederic Bois.  All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -19,19 +19,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-   contact
-     Frdric Bois / Don Maszle
-     BEHS, School of Public Health
-     University of California at Berkeley
-     Berkeley, CA 94720
-
-     fbois@diana.lbl.gov
-
    -- Revisions -----
-     Logfile:  SCCS/s.modelu.c
-    Revision:  1.25
-        Date:  14 Nov 1997
-     Modtime:  06:39:04
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -106,14 +98,11 @@ BOOL vbModelReinitd = FALSE;    /* Model has been initialized */
    point, the input has decayed to a neglible quantity, and we don't
    have to waste time calling the exp() function.
 
-   After N_TAU_EXPOSE == 10 Tau's, the input has decayed to 0.0045 %
-   of the magnitude.  For some reason this causes a numerical error.
-
    After N_TAU_EXPOSE == 40 Tau's, the input has decayed to 4.2e-18
    of the magnitude.
 */
 
-void FixupDependentInputs()
+void FixupDependentInputs (void)
 {
   int i, j;
 
@@ -158,7 +147,7 @@ void FixupDependentInputs()
       vrgInputs[i].dTexp = N_TAU_EXPOSE / vrgInputs[i].dDecay;
       if (vrgInputs[i].dTper < vrgInputs[i].dTexp)
         vrgInputs[i].dTexp = vrgInputs[i].dTper;
-    } /* if */
+    }
 
     /* FB 9/9/97: Assure that input does not exceed the period */
     if (vrgInputs[i].dTexp != 0.0
@@ -198,7 +187,7 @@ void FixupDependentInputs()
 void GetStartPeriods (PDOUBLE pdTime)
 {
   int i;
-  double dTmp;
+  double dTmp, dDummy;
 
   if (*pdTime == 0.0)
     for (i = 0; i < vnInputs; i++) {
@@ -228,8 +217,9 @@ void GetStartPeriods (PDOUBLE pdTime)
           vrgInputs[i].dTStartPeriod = 0.0;
         else {
           dTmp = (*pdTime / vrgInputs[i].dTper);
-          vrgInputs[i].dTStartPeriod = vrgInputs[i].dTper *
-                                       (dTmp - (long) dTmp);
+          /* FB 6/4/99 introduced modf */
+          vrgInputs[i].dTStartPeriod = vrgInputs[i].dTper * 
+                                       modf (dTmp, &dDummy); 
         } /* else */
       } /* else */
     } /* for */
@@ -312,7 +302,7 @@ BOOL UpdateSpikes (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
 /* -----------------------------------------------------------------------------
    UpdateDefaultInput
 
-   Update default input types.  PerRate, PerDose, PerExp.
+   Update default input types : PerDose, PerExp.
 */
 void UpdateDefaultInput (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
 {
@@ -325,10 +315,9 @@ void UpdateDefaultInput (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
 
     if (!pifn->bOn) {
       if (pifn->dTper != 0.0) /* Periodic */
-    *pdTnext = (pifn->dTStartPeriod += pifn->dTper); /* Next period */
-
+        *pdTnext = (pifn->dTStartPeriod += pifn->dTper); /* Next period */
       else
-    *pdTnext = pifn->dTStartPeriod = DBL_MAX - pifn->dTper;
+        *pdTnext = pifn->dTStartPeriod = DBL_MAX - pifn->dTper;
     } /* if */
   } /* if */
 
@@ -394,7 +383,6 @@ void UpdateInputs (PDOUBLE pdTime, PDOUBLE pdNextTransTime)
 
       default:
       case IFN_PERDOSE:
-      case IFN_PERRATE:
       case IFN_PEREXP:
         if (vrgInputs[i].dMag != 0.0)
           UpdateDefaultInput (&vrgInputs[i], &dT, pdTime);
@@ -444,6 +432,7 @@ HVAR GetVarHandle (PSTR szName)
   PVMMAPSTRCT pvm = GetVarPtr (vrgvmGlo, szName);
 
   return (pvm ? pvm->hvar : ID_NULL);
+
 } /* GetVarHandle */
 
 
@@ -508,6 +497,7 @@ static char szInvalid[] = "InvalidVariable?";
     pvm++;
 
   return (*pvm->szName ? pvm->szName : szInvalid);
+
 } /* GetVarType */
 
 
@@ -559,6 +549,7 @@ double GetVarValue (HVAR hvar)
 BOOL IsInput (HVAR hvar)
 {
   return (GetVarType(hvar) == ID_INPUT);
+
 } /* IsInput */
 
 /* -----------------------------------------------------------------------------
@@ -570,6 +561,7 @@ BOOL IsInput (HVAR hvar)
 BOOL IsState (HVAR hvar)
 {
   return (GetVarType(hvar) == ID_STATE);
+
 } /* IsState */
 
 
@@ -582,6 +574,7 @@ BOOL IsState (HVAR hvar)
 BOOL IsOutput (HVAR hvar)
 {
   return (GetVarType(hvar) == ID_OUTPUT);
+
 } /* IsOutput */
 
 
@@ -595,6 +588,7 @@ BOOL IsModelVar (HVAR hvar)
 {
   int iType = GetVarType(hvar);
   return (iType == ID_STATE || iType == ID_OUTPUT);
+
 } /* IsState */
 
 
@@ -620,6 +614,7 @@ BOOL IsParm (HVAR hvar)
 int ModelIndex (HVAR hvar)
 {
   return (IsModelVar(hvar) ? HINDEX(hvar) : 0);
+
 } /* ModelIndex */
 
 
@@ -655,6 +650,7 @@ BOOL SetVar (HVAR hvar, double dVal)
   } /* switch */
 
   return (bReturn);
+
 } /* SetVar */
 
 /* -----------------------------------------------------------------------------
@@ -672,6 +668,7 @@ BOOL SetInput (HVAR hvar, PIFN pifn)
 
   memcpy (&vrgInputs[HINDEX(hvar)], pifn, sizeof(IFN));
   return (TRUE);
+
 } /* SetVar */
 
 
@@ -684,6 +681,7 @@ BOOL SetInput (HVAR hvar, PIFN pifn)
 PDOUBLE GetModelVector (void)
 {
   return ((PDOUBLE) vrgModelVars);
+
 } /* GetModelVector */
 
 /* -----------------------------------------------------------------------------
@@ -696,6 +694,7 @@ PDOUBLE GetModelVector (void)
 int GetNModelVars (void)
 {
   return (vnModelVars);
+
 } /* GetNModelVars */
 
 /* -----------------------------------------------------------------------------
@@ -708,6 +707,7 @@ int GetNModelVars (void)
 int GetNStates (void)
 {
   return (vnStates);
+
 } /* GetNStates */
 
 
@@ -747,13 +747,13 @@ void CalcInputs (PDOUBLE pdTime)
                                 vrgInputs[i].dMag; /* Scaled */
           break;
 
-        case IFN_PERDOSE:
-        case IFN_PERRATE: /* Does not change step-wise */
+        case IFN_PERDOSE: /* Does not change step-wise */
           break;
 
         case IFN_PEREXP: /* Calc current exp() */
-          vrgInputs[i].dVal = vrgInputs[i].dMag * 
-                              exp ((vrgInputs[i].dTStartPeriod - *pdTime) * 
+          vrgInputs[i].dVal = vrgInputs[i].dMag * ((int) vrgInputs[i].bOn) *
+                              exp ((vrgInputs[i].dTStartPeriod + 
+                                    vrgInputs[i].dT0 - *pdTime) * 
                                    vrgInputs[i].dDecay);
           break;
 
