@@ -3,7 +3,7 @@
    written by Don Maszle
    7 October 1991
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1991-2004.  Don Maszle, Frederic Bois.  All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -19,19 +19,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-   contact
-     Frdric Bois / Don Maszle
-     BEHS, School of Public Health
-     University of California at Berkeley
-     Berkeley, CA 94720
-
-     fbois@diana.lbl.gov
-
    -- Revisions -----
-     Logfile:  SCCS/s.modelu.h
-    Revision:  1.10
-        Date:  14 Nov 1997
-     Modtime:  06:39:17
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -46,12 +38,12 @@
 
 #ifndef _MODELU_H_
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    Inclusions  */
 
 #include "modiface.h"
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    Constants  */
 
 #define ID_NULL     0x0000
@@ -64,7 +56,7 @@
 #define HV_INDEX_MASK  0x0FFF /* == 0xtiii, type and index */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    Typedefs  */
 
 /* Global Variable Map */
@@ -76,7 +68,7 @@ typedef struct tagVM {
 } VMMAPSTRCT, *PVMMAPSTRCT; /* Variable Map element */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    Macros  */
 
 #define TYPE(pvm)    ((pvm) ? (pvm)->hvar & HV_TYPE_MASK : ID_NULL)
@@ -86,10 +78,10 @@ typedef struct tagVM {
 #define HINDEX(hvar) ((int) ((hvar) & HV_INDEX_MASK))
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    Prototypes  */
 
-void FixupDependentInputs(void);
+void FixupDependentInputs (void);
 
 void GetStartPeriods (PDOUBLE pdTime);
 void GetStateHandles (HVAR *phvar);
