@@ -4,7 +4,7 @@
    modifications by Don Robert Maszle
    8 January 1992
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1992-2004.  Don Maszle, Frederic Bois.  All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -20,19 +20,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-   contact
-     Frdric Bois / Don Maszle
-     BEHS, School of Public Health
-     University of California at Berkeley
-     Berkeley, CA 94720
-
-     fbois@diana.lbl.gov
-
    -- Revisions -----
-     Logfile:  SCCS/s.random.c
-    Revision:  1.29
-        Date:  14 Nov 1997
-     Modtime:  06:39:06
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -59,6 +51,7 @@
    PiecewiseRandom (min, a, b, max)            -- Draws from a mayan pyramid !
    PiecewiseVariate (n, x[], p[], Cdf[], o, p) -- Draws from a tabulated PDF
    PoissonRandom(mu)                  -- Poisson with rate mu
+   TruncInvGGammaRandom (alpha, beta, a, b)    -- Truncated general inv-gamma
    TruncLogNormalRandom (m, s, a, b)  -- Truncated log normal
    TruncNormalRandom (m, s, a, b)     -- Truncated general normal
    UniformRandom(a, b)                -- Uniform over [a, b]
@@ -68,7 +61,7 @@
 
    CDFNormal(z)                                -- Integral of the normal at z
    CalcCumulative (n, x[], p[], Cdf[], o)      -- Constructs a CDF given a PDF
-   erfc(x)                                     -- Error function
+   erfc(x)                                     -- Complementary error function
    lnDFNormal(x, mu, sd)                       -- Log of the normal density
    lnGamma(x)                                  -- Natural log of gamma function
    lnDFBeta (x, alpha, beta, min, max)         -- Log of beta density
@@ -107,11 +100,30 @@ typedef struct tagRANDREC {
 } RANDREC, *PRANDREC;
 
 
-static vbNoSeed = TRUE;         /* Flag to prevent use without seed */
-static vbNotInitd = TRUE;       /* Flag to prevent use without initializing */
+static BOOL vbNoSeed = TRUE;    /* Flag to prevent use without seed */
+static BOOL vbNotInitd = TRUE;  /* Flag to prevent use without initializing */
 static RANDREC vRandRec;        /* Global random informatino shared by */
                                 /* all random number functions */
 
+
+/* ----------------------------------------------------------------------------
+   GetSeed
+
+   Returns the current value of vRandRec.seed.
+*/
+
+double GetSeed (void)
+{
+  return (vRandRec.seed);
+}
+
+
+/* ----------------------------------------------------------------------------
+   SetSeed
+
+   Sets vRandRec.seed to given dSeed, silently corrects an invalid dSeed.
+
+*/
 void SetSeed (double dSeed)
 {
   int bCorrected = 0;
@@ -149,7 +161,7 @@ void SetSeed (double dSeed)
 } /* SetSeed */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    InitRandom
 
    initializes the random generator with the given seed.
@@ -160,7 +172,6 @@ void SetSeed (double dSeed)
    array is filled from which shuffled random values will be drawn.
    Also, a flag used by the Normal() routine is initialized.
 */
-
 void InitRandom (double dSeed, int bWarmUp)
 {
   long i;
@@ -184,7 +195,7 @@ void InitRandom (double dSeed, int bWarmUp)
 } /* InitRandom */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    Randoms
 
    An alternative random number generator, so you don't have to use
@@ -232,7 +243,7 @@ double Randoms (void)
 } /* Randoms */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    RandomShuffle
 
    assumes that the random number generator is Randoms ().
@@ -261,7 +272,7 @@ double RandomShuffle (void)
 } /* RandomShuffle */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    NormalRandom
 
    Returns a Normal random variate based on a unit variate,
@@ -299,7 +310,7 @@ double NormalRandom (double dMean, double dStdDev)
 } /* NormalRandom */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    LogNormalRandom
 
    returns a variate such that the log of the variate is normally
@@ -328,7 +339,7 @@ double LogNormalRandom (double dMean, double dStdDev)
 } /* LogNormalRandom */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    TruncNormalRandom
 
    returns a truncated Normal variate in the range [a, b].
@@ -358,7 +369,7 @@ double TruncNormalRandom (double dMean, double dStdDev, double a, double b)
 } /* TruncNormalRandom */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    TruncLogNormalRandom
 
    returns a truncated LogNormal variate in the range [a, b].
@@ -451,7 +462,7 @@ double GammaRandom (double alpha)
         x = -log((b - u1) / alpha);
         if (pow(x, alpha - 1) >= Randoms()) return(x);
       }
-    } while (1);
+    } while (1 == 1);
 
   } /* end if alpha < 1 */
 
@@ -482,7 +493,7 @@ double GammaRandom (double alpha)
         if (((c3 * u1 + w + 1 / w) <= c4) ||
             ((c3 * log(u1) - log(w) + w) < 1))
           return(c1 * w);
-      } while (1);
+      } while (1 == 1);
 
     }
     else
@@ -517,7 +528,7 @@ double GGammaRandom (double alpha, double beta)
 /* ----------------------------------------------------------------------------
    InvGGammaRandom
 
-   Returns an inverse gamma distributed random variate with shaping parameter
+   Returns an inverse gamma distributed random variate with shape parameter
    alpha and scale parameter beta.
    This just gets a general gamma variate and returns its inverse
    See Gelman et al. "Bayesian Data Analysis"
@@ -538,6 +549,34 @@ double InvGGammaRandom (double alpha, double beta)
 } /* InvGGammaRandom */
 
 
+/* ----------------------------------------------------------------------------
+   TruncInvGGammaRandom
+
+   returns a truncated general inverse gamme variate in the range [a, b].
+*/
+
+double TruncInvGGammaRandom (double alpha, double beta, double a, double b)
+{
+  double X = 0.0;
+  int    iter = 0;
+
+  if (a >= b)
+    printf ("TruncLogNormalRandom: min >= max  [%g %g]\n", a, b);
+
+  else do {
+    if(++iter == 25) {
+      printf("TruncInvGGammaRandom: problem with range: ");
+      printf("min %g, max %g, alpha %g, beta %g\n", a, b, alpha, beta);
+    }
+    X = InvGGammaRandom(alpha, beta);
+  }
+  while (X < a || X > b);
+
+  return X;
+
+} /* TruncLogNormalRandom */
+
+
 /* ----------------------------------------------------------------------------
    Chi2Random
 
@@ -633,7 +672,7 @@ double BinomialRandom (double p, long N)
   static double dLnFactN, dPold = -1, dLnP, dQ, dLnQ;
 
   if (p < 0 || p > 1 || N < 0) {
-    printf ("Error: negative or null parameters for a binomial variate "
+    printf ("Error: parameters out of bounds for a binomial variate "
             "- Exiting\n\n");
     exit (0);
   }
@@ -951,7 +990,7 @@ double BetaRandom (double alpha, double beta, double a, double b)
 /* -----------------------------------------------------------------------------
    erfc
 
-   the error function of z.
+   the complementary error function of z (= 1 - erf(z))
 
    Adapted from the algorithm described in the book Numerical Recipes by
    Press et al. 
@@ -963,11 +1002,16 @@ double erfc (double x)
   double dAbsX, t, dVal;
 
   dAbsX = fabs(x);
-  t = 1 / (1 + 0.5 * dAbsX);
-  dVal = t * exp(-dAbsX*dAbsX - 1.26551223 + t*(1.00002368 + t*(0.37409196 +
-         t*(0.09678418 + t*(-0.18628806 + t*(0.27886807 + t*(-1.13520398 +
-         t*(1.48851587 + t*(-0.82215223 + t*(0.17087277))))))))));
-  return ( x >= 0 ? dVal : 2 - dVal );
+  if (dAbsX > 20) { /* FB 16/2/99) */
+    return ( x >= 0 ? 0 : 2 );
+  }
+  else {
+    t = 1 / (1 + 0.5 * dAbsX);
+    dVal = t * exp(-dAbsX*dAbsX - 1.26551223 + t*(1.00002368 + t*(0.37409196 +
+           t*(0.09678418 + t*(-0.18628806 + t*(0.27886807 + t*(-1.13520398 +
+           t*(1.48851587 + t*(-0.82215223 + t*(0.17087277))))))))));
+    return ( x >= 0 ? dVal : 2 - dVal );
+  }
 }
 
 
@@ -979,7 +1023,16 @@ double erfc (double x)
 
 double CDFNormal (double z)
 {
-  return ( 0.5 * (2 - erfc(z/sqrt(2))) );
+  register double tmp;
+
+  tmp = z/SQRT_2;
+
+  /* avoid roundoff errors generated by subtracting a small number from
+     a much bigger one */
+  if (tmp >= 0)
+    return ( 0.5 * (2 - erfc(tmp)) );
+  else
+    return ( 0.5 * erfc(-tmp));
 }
 
 
