@@ -4,7 +4,7 @@
    modifications by Don Robert Maszle
    8 January 1992
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1992-2004.  Don Maszle, Frederic Bois.  All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -20,19 +20,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-   contact
-     Frdric Bois / Don Maszle
-     BEHS, School of Public Health
-     University of California at Berkeley
-     Berkeley, CA 94720
-
-     fbois@diana.lbl.gov
-
    -- Revisions -----
-     Logfile:  SCCS/s.random.h
-    Revision:  1.21
-        Date:  14 Nov 1997
-     Modtime:  06:39:19
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -54,6 +46,7 @@
 #define SEED_DEFAULT 314159265.3589793
 #define PI           3.1415926535897932384626433
 #define INV_SQRT_2PI 0.398942280401433
+#define SQRT_2       1.4142135623731
 
 
 /* ----------------------------------------------------------------------------
@@ -78,6 +71,7 @@ double BinomialRandom (double p, long n);
 double Chi2Random (double dof);
 double ExpRandom (double beta);
 double InvGGammaRandom (double alpha, double beta);
+double TruncInvGGammaRandom (double alpha, double beta, double a, double b);
 double GammaRandom (double alpha);
 double GGammaRandom (double alpha, double beta);
 double LogNormalRandom (double dMean, double dStdDev);
@@ -97,10 +91,11 @@ void   WishartRandom (long n, long p, double *t, double *w, double *work);
 /* ----------------------------------------------------------------------------
    Utility functions */
 
-double CDFNormal(double z);
+double CDFNormal (double z);
 double InterpolateX (double rgX[], double rgY[], long lLower, double dY);
-double erfc(double x);
-double lnDFNormal(double x, double mu, double sd);
+double erfc (double x);
+double GetSeed (void);
+double lnDFNormal (double x, double mu, double sd);
 double lnGamma (double x);
 double lnDFBeta (double x, double alpha, double beta, double min, double max);
 void   CalcCumulative (long cDim, double *rg_x, double *rg_pdf,
