@@ -1,6 +1,6 @@
 /* sim.c
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1993-2004.  Don Maszle, Frederic Bois.  All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -16,19 +16,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-   contact
-     Frdric Bois / Don Maszle
-     BEHS, School of Public Health
-     University of California at Berkeley
-     Berkeley, CA 94720
-
-     fbois@diana.lbl.gov
-
    -- Revisions -----
-     Logfile:  SCCS/s.sim.c
-    Revision:  1.48
-        Date:  14 Nov 1997
-     Modtime:  06:39:06
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -36,11 +28,6 @@
 
 */
 
-#ifdef _MACOSLEVEL2_
-#include <PPC Macheaders>
-#include "mac.h"
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -51,11 +38,11 @@
 #include "yourcode.h"
 #include "getopt.h"
 #include "mh.h"
-#include "optdesign.h"
+#include "optdsign.h"
 #include "lexerr.h"
 #include "lsodes.h"
 #include "sim.h"
-/* #include "simhelp.h" */
+/* #include "simhelp.h" */
 #include "simi.h"
 #include "siminit.h"
 #include "simo.h"
@@ -63,10 +50,10 @@
 #include "strutil.h"
 
 
-static char vszVersion[] = "v4.2.0"; /* Version of program */
+static char vszVersion[] = "v5.0.0"; /* Version of program */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    CorrectInputToTransition
 
    resets the integrator and inputs when an input transition occurs.
@@ -88,7 +75,7 @@ void CorrectInputToTransition (PEXPERIMENT pexp, PDOUBLE pdTtrans)
 } /* CorrectInputToTransition */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    Euler
 
    Simple Euler integrator.
@@ -137,7 +124,7 @@ int Euler (long neq, double *y, double *t, double tout, double dTStep)
 } /* Euler */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    FreeVarMod
 
    Callback for FreeList().
@@ -160,7 +147,7 @@ void FreeVarMod (PVOID pData)
 } /* FreeVarMod */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    ModifyOneParm
 
    Callback function for ModifyParms.
@@ -180,7 +167,7 @@ int ModifyOneParm (PVOID pData, PVOID pNullInfo)
 } /* ModifyOneParm */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    ModifyParms
 
    Modifies the parameters in the plistParmMods LIST of the experiment
@@ -356,7 +343,7 @@ int DoOneNormalExp (PANALYSIS panal, PEXPERIMENT pexp)
    Each Run
     calc mc mods
 
-     Each Experiment
+     Each Simulation (old "Experiment")
      1)  Init the model
      2)  Global parm mods
      3)  Monte Carlo mods
@@ -390,7 +377,7 @@ int DoOneMCExp (PANALYSIS panal, PEXPERIMENT pexp)
 } /* DoOneMCExp */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    DoNormal
 
    Does a normal analysis
@@ -509,7 +496,7 @@ void DoMonteCarlo (PANALYSIS panal)
 } /* DoMonteCarlo */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    DoAnalysis
 
    Does the analysis in the given specification.
@@ -540,15 +527,15 @@ void DoAnalysis (PANALYSIS panal)
 
   } /* switch */
 
-  if (panal->expGlobal.os.pfileOut) {
-    fclose (panal->expGlobal.os.pfileOut);
-    printf ("Wrote output file \"%s\"\n", panal->expGlobal.os.szOutfilename);
+  if (panal->pfileOut) {
+    fclose (panal->pfileOut);
+    printf ("Wrote output file \"%s\"\n", panal->szOutfilename);
   }
 
 } /* DoAnalysis */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    MCVarListToArray
 
    converts a list of MCVAR to an array.  This must be a callback for
@@ -600,7 +587,7 @@ void PrepAnalysis (PANALYSIS panal)
   ForAllList (pmc->plistMCVars, MCVarListToArray, (PVOID) NULL);
   FreeList (&pmc->plistMCVars, NULL, FALSE);
 
-  /* Make a handle vector for theta */
+  /* Make a handle vector */
   pmc->rghvar = (HVAR *) malloc((pmc->nParms)*sizeof(HVAR));
   if (pmc->rghvar) {
     for (l = 0; l < pmc->nParms; l++)
@@ -614,7 +601,7 @@ void PrepAnalysis (PANALYSIS panal)
 
 /* Get the command line argument stuff */
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    SansPath
 
    returns a pointer to just the filename of a full path.
@@ -636,7 +623,7 @@ char *SansPath (char *szFullPathname)
 } */ /* SansPath */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    PromptFilenames
 
    prompts for both input and output file names.  The space allocated
@@ -645,16 +632,16 @@ char *SansPath (char *szFullPathname)
 
 void PromptFilenames (PSTR *pszFileIn, PSTR *pszFileOut)
 {
-  *pszFileIn = (PSTR) calloc (1, 80);
-  *pszFileOut = (PSTR) calloc (1, 80);
+  *pszFileIn  = (PSTR) calloc (1, MAX_FILENAMESIZE);
+  *pszFileOut = (PSTR) calloc (1, MAX_FILENAMESIZE);
 
   printf ("Input filename? ");
 #ifdef _MACOSLEVEL1_
   scanf ("%[^:\f\r\v\n]", *pszFileIn);
   getchar();
 #else
-  gets (*pszFileIn);
-  *pszFileIn = strtok (*pszFileIn, " \t");
+  fgets (*pszFileIn, MAX_FILENAMESIZE, stdin);
+  *pszFileIn = strtok (*pszFileIn, " \t\n");
 #endif
 
   if (!(*pszFileIn)) /* Nothing entered, quit */
@@ -665,8 +652,8 @@ void PromptFilenames (PSTR *pszFileIn, PSTR *pszFileOut)
 #ifdef _MACOSLEVEL1_
     scanf ("%[^:\f\r\v\n]", *pszFileOut);
 #else
-    gets (*pszFileOut);
-    *pszFileOut = strtok (*pszFileOut, " \t");
+    fgets (*pszFileOut, MAX_FILENAMESIZE, stdin);
+    *pszFileOut = strtok (*pszFileOut, " \t\n");
 #endif
   }
 
@@ -682,7 +669,7 @@ void PromptFilenames (PSTR *pszFileIn, PSTR *pszFileOut)
 } /* PromptFilenames */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    Command Line Parsing
 
 */
@@ -703,7 +690,7 @@ void GetOutputFlagOption (PANALYSIS panal, char *optarg)
 } */ /* GetOutputFlagOption */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    GetFilenames
 
    retrieves options and filenames from the command line arguments passed to
@@ -810,13 +797,13 @@ void GetCmdLineArgs (int cArg, char *const *rgszArg, PSTR *pszFileIn,
 } /* GetCmdLineArgs */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
 */
 void AnnounceProgram (void)
 {
   printf ("\n________________________________________\n");
   printf ("\nMCSim %s\n\n", vszVersion);
-  printf ("Copyright (c) 1993-1997 by F. Bois & D. Maszle. "
+  printf ("Copyright (c) 1993-2004 by F. Bois & D. Maszle. "
           "All rights reserved.\n\n");
 
   printf ("MCSim comes with ABSOLUTELY NO WARRANTY;\n"
@@ -829,7 +816,7 @@ void AnnounceProgram (void)
 } /* AnnounceProgram */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    main
 
    Entry point for simulation and analysis program.
@@ -856,9 +843,9 @@ int main (int nArg, char *const *rgszArg)
   GetCmdLineArgs (nArg, rgszArg, &szFileIn, &szFileOut, panal);
 
   /* Define the output file as the global experiment default  */
-  panal->expGlobal.os.szOutfilename = szFileOut;
-  szFileOut == NULL ? (panal->expGlobal.os.bCommandLineSpec = FALSE) :
-                      (panal->expGlobal.os.bCommandLineSpec = TRUE);
+  panal->szOutfilename = szFileOut;
+  szFileOut == NULL ? (panal->bCommandLineSpec = FALSE) :
+                      (panal->bCommandLineSpec = TRUE);
 
   if (!InitBuffer (&ibIn, szFileIn))
     ReportError (&ibIn, RE_INIT | RE_FATAL, "ReadInput", NULL);
