@@ -2,7 +2,7 @@
 
    originally written by Don Maszle
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1993-2004.  Don Maszle, Frederic Bois.  All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -18,19 +18,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-   contact
-     Frdric Bois / Don Maszle
-     BEHS, School of Public Health
-     University of California at Berkeley
-     Berkeley, CA 94720
-
-     fbois@diana.lbl.gov
-
    -- Revisions -----
-     Logfile:  SCCS/s.sim.h
-    Revision:  1.35
-        Date:  14 Nov 1997
-     Modtime:  06:39:20
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -60,19 +52,20 @@
    Other maximum sizes are MAX_EQN and MAX_LEX in lex.h */
 
 #define MAX_LEVELS            10   /* for now; actual possible is 255 */
-#define MAX_INSTANCES         200  /* for now; allowable instances of `Level'
-                                      at any depth */
-#define MAX_EXPERIMENTS       200
+#define MAX_INSTANCES         200  /* allowable instances of `Level' at any
+                                      depth and max number of experiments */
 #define LSODES_IWORKSIZE      300  /* Lsodes tells if this is not big enough */
-#define LSODES_RWORKSIZE      1200 /* Idem */
+#define LSODES_RWORKSIZE      6000 /* Idem */
 #define ARGS_MAX              8    /* Maximum number of args to lex */
                                    /* Used in allocating argument strings */
 #define MAX_PRINT_VARS        10   /* Arbitrary */
+#define MAX_FILENAMESIZE      80   /* Arbitrary */
 
 /* Keyword Map constants */
 
 #define KM_INTEGRATE       1
 #define KM_SIMULATE        2
+#define KM_STARTTIME       3
 #define KM_PRINT           5
 #define KM_PRINTSTEP       6
 #define KM_DATA            7
@@ -115,6 +108,8 @@
 #define KM_TRUNCNORMALV    226
 #define KM_TRUNCLOGNORMALV 227
 #define KM_BINOMIALBETA    228
+#define KM_HALFNORMAL      229
+#define KM_TRUNCINVGGAMMA  230
 
 #define KM_PREDICTION      300
 
@@ -141,7 +136,7 @@
 #define AT_MONTECARLO   2    /* Monte Carlo variations */
 #define AT_SETPOINTS    3    /* Set points simulation */
 #define AT_MCMC         4    /* Metropolis or Gibbs estimation */
-#define AT_OPTDESIGN    5    /* Metropolis or Gibbs estimation */
+#define AT_OPTDESIGN    5    /* Optimal design search */
 
 /* Monte Carlo Variation types */
 
@@ -165,6 +160,8 @@
 #define MCV_TRUNCNORMALV    16
 #define MCV_TRUNCLOGNORMALV 17
 #define MCV_BINOMIALBETA    18
+#define MCV_HALFNORMAL      19
+#define MCV_TRUNCINVGGAMMA  20
 
 /* Integration Method types */
 
@@ -190,8 +187,10 @@
 
 /* Defs for Distrib statement */
 
-#define MCVP_FIXED    0
-#define MCVP_VARIABLE 1
+#define MCVP_FIXD     0
+#define MCVP_PARM     1
+#define MCVP_PRED     2
+#define MCVP_DATA     3
 
 
 /* ----------------------------------------------------------------------------
@@ -257,28 +256,30 @@ typedef struct tagDATAREC {
 /* Output specification */
 
 typedef struct tagOUTSPEC {
-  PSTR  szOutfilename;      /* Name of file for regular output */
-  PFILE pfileOut;           /* Pointer to file */
-  BOOL  bCommandLineSpec;   /* Output file specified on command line */
+  int     nOutputs;           /* Number of outputs statements */
+  PLIST   plistPrintRecs;     /* List of records from Print()'s */
+  PSTR    *pszOutputNames;    /* Array of output names */
+  HVAR    *phvar_out;         /* Array of handles to outputs */
 
-  int   nOutputs;           /* Number of outputs */
-  PSTR  *pszOutputNames;    /* Array of output names */
-  HVAR  *phvar;             /* Array of handles to outputs */
-
-  PLIST plistPrintRecs;     /* List of records from Print()'s */
-  PLIST plistDataRecs;      /* List of records from Data()'s */
+  int     nData;              /* Number of data statements */
+  PLIST   plistDataRecs;      /* List of records from Data()'s */
+  PSTR    *pszDataNames;      /* Array of output names */
+  HVAR    *phvar_dat;         /* Array of handles to outputs */
 
   /* The lists are converted into the following */
 
-  PINT    pcOutputTimes;    /* Count of output times for each var */
-  PINT    piCurrentOut;     /* Index to current output for each var */
-  PINT    piSigmas;         /* Array of SDs for each var */
-  PDOUBLE *prgdOutputTimes; /* Array of output times for each var */
-  PDOUBLE *prgdOutputVals;  /* Array of output values for each var */
-  PDOUBLE *prgdDataVals;    /* Array of data values for each var */
+  PINT    pcOutputTimes;    /* Count of output times for each output */
+  PINT    piCurrentOut;     /* Index to current output for each output */
+  PDOUBLE *prgdOutputTimes; /* Array of output times for each output */
+  PDOUBLE *prgdOutputVals;  /* Array of output values for each output */
 
-  int cDistinctTimes;       /* Count of distinct output times */
-  PDOUBLE rgdDistinctTimes; /* Array of distinct output times */
+  int     cDistinctTimes;   /* Count of distinct output times for all
+                               outputs */
+  PDOUBLE rgdDistinctTimes; /* Array of distinct output times  for all
+                               outputs */
+
+  PINT    pcData;           /* Count of values for each data */
+  PDOUBLE *prgdDataVals;    /* Array of values for each data */
 
 } OUTSPEC, *POUTSPEC; /* tagOUTSPEC */
 
@@ -288,23 +289,29 @@ typedef struct tagOUTSPEC {
 typedef struct tagMCVAR {
   PSTR    pszName;          /* Model variableariable name */
   HVAR    hvar;             /* Handle to the model variable to be modified */
+  double  dVal;             /* Current value */
+  PDOUBLE pdVal;            /* Pointer to value */
+
   int     iDepth;           /* Level (to distinguish vars with same hvar) */
   int     iType;            /* One of MCV_ distribution types */
-  char    cVarParm;         /* Flag for variable parameters */
-  HVAR    hParm[4];         /* pointers to model vars for 4 distrib. params */
-  double  dParm[4];         /* values of distribution parameters */
-  PDOUBLE pdParm[4];        /* pointers to distribution parameters */
-  double  dVal;             /* Value for this run */
+
+  HVAR    hParm[4];         /* Handles to model vars for 4 distrib. params */
+  double  dParm[4];         /* Values of fixed distribution parameters */
+  PDOUBLE pdParm[4];        /* Pointers to symbolic distribution parameters */
+  int     iParmType[4];     /* distrib. parameter types (const., param., 
+                               pred., or data) */
+
   struct  tagMCVAR *pMCVParent[4]; /* Pointers to parents of this var (vars on
                                       which this var depends) */
   PLIST   plistDependents;  /* List of MCvars depending directly on this one */
   long    nDependents;
   struct  tagMCVAR **rgpDependents;
+
   BOOL    bExptIsDep;       /* True if experiment is dependent on this var */
-  int     iParmIndex;       /* Index in OUTSPEC when Parm[1] is ModelVar */
   BOOL    bIsFixed;         /* True if var is fixed */
   BOOL    bGibbs;           /* True if its conditional distrib. is known */
   long    lJumps;           /* Number of MH jumps for this param */
+  long    lCount;           /* Number of data values eventually attached */
   double  dKernelSD;        /* MCMC jumping kernel SD */
 
 } MCVAR, *PMCVAR; /* tagMCVAR */
@@ -348,9 +355,9 @@ typedef struct tagMONTECARLO {
   HVAR   *rghvar;           /* Array of handles to the parameters */
   MCVAR  **rgpMCVar;        /* A priori distributions for each */
 
-  long nSetParms;           /* Count of setpoint parameters */
+  long   nSetParms;         /* Count of setpoint parameters */
 
-  int  style;               /* either forward or backward for optimal design */
+  int    style;             /* either forward or backward for optimal design */
 
 } MONTECARLO, *PMONTECARLO; /* tagMONTECARLO */
 
@@ -378,19 +385,17 @@ typedef struct tagEXPERIMENT {
 
   double dT0;               /* Time limits */
   double dTfinal;
-
-  double dTime;             /* Current Time -- not used for global sim */
-
-  int iSubject;             /* Subject #, eventually */
-
-  double dLnLike;           /* Log-likelihood */
-  double dLnLikeSave;
+  double dTime;             /* Current Time */
 
   PMODELINFO pmodelinfo;    /* Pointer to the model information */
   PLIST plistParmMods;      /* List of parameter mods (elt = PVARMOD) */
+
   INTSPEC is;               /* Integrator spec, this experiment */
   OUTSPEC os;               /* Output spec, this experiment */
 
+  double dLnLike;           /* Log-likelihood */
+  double dLnLikeSave;
+
 } EXPERIMENT, *PEXPERIMENT; /* tagEXPERIMENT */
 
 
@@ -402,16 +407,21 @@ typedef struct tagLEVEL {
   int    iInstances;                       /* # of instances of next level */
   struct tagLEVEL *pLevels[MAX_INSTANCES]; /* Pointers to sublevels */
 
-  PLIST  plistVars;    /* Vars, other than those in Distrib */
-  long   nFixedVars;
-  PVARMOD *rgpFixedVars;
+  PLIST   plistVars;     /* Vars, other than those in Distrib */
+  long    nFixedVars;    /* Count of fixed parameters */
+  PVARMOD *rgpFixedVars; /* Array of var values (from plistVars) */
 
-  PLIST   plistMCVars; /* List of MCVAR record, variation specs */
+  PLIST   plistMCVars; /* List of MCVAR records, variation specs */
   long    nMCVars;     /* Count of parameters */
-  PMCVAR *rgpMCVars;   /* A priori distributions for each */
+  PMCVAR *rgpMCVars;   /* Array of MCVAR records (from plistMCVars) */
+
+  PLIST   plistLikes;  /* List of MCVAR records, likelihood specs */
+  long    nLikes;      /* Count of likelihoods */
+  PMCVAR *rgpLikes;    /* Array of MCVAR records (from plistLikes) */
 
   PEXPERIMENT pexpt;   /* Ptr to expt struct, NULL if not expt
-				                  EXPERIMENT is used for compatibility */ 
+			  EXPERIMENT is used for compatibility */ 
+
 } LEVEL, *PLEVEL; /* tagLEVEL */
 
 
@@ -420,7 +430,7 @@ typedef struct tagLEVEL {
 typedef struct tagANALYSIS {
 
   BOOL bDependents;	    /* Debug flag for printing dependents to stderr */
-  BOOL bParams;		      /* Debug flag for printing params of MC vars to file */
+  BOOL bParams;		    /* Debug flag for printing params of MC vars */
   BOOL bPrintIter;	    /* Debug flag for printing iteration numbers */
 
   int iType;            /* Type of analysis. One of AT_ types */
@@ -442,22 +452,22 @@ typedef struct tagANALYSIS {
 
   EXPERIMENT  expGlobal;            /* Global experiment settings */
 
-  PEXPERIMENT rgpExps[MAX_EXPERIMENTS];/* List of pointer to experiments */
+  PSTR  szOutfilename;      /* Name of file for regular output */
+  PFILE pfileOut;           /* Pointer to file */
+  BOOL  bCommandLineSpec;   /* Output file specified on command line */
+
+  PEXPERIMENT rgpExps[MAX_INSTANCES];  /* List of pointer to experiments */
   PEXPERIMENT pexpCurrent;             /* Experiment being currently defined */
 
   PLIST plistVars;	    /* Global variables to set */
 
-  PLIST plistModelVars; /* List of output variables to randomize after expt */
-  long nModelVars;
-  PMCVAR *rgpModelVars;
-
-  MONTECARLO    mc;     /* Monte Carlo specification */
-  GIBBSDATA     gd;     /* Gibbs estimate specification */
+  MONTECARLO    mc;     /* Monte Carlo specification data */
+  GIBBSDATA     gd;     /* MCMC specification data */
 
 } ANALYSIS, *PANALYSIS; /* tagANALYSIS */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    Globals/Externals
 */
 
