@@ -2,7 +2,7 @@
 
    originally written by Frederic Bois
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1993-2004.  Don Maszle, Frederic Bois.  All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -18,19 +18,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-   contact
-     Frdric Bois / Don Maszle
-     BEHS, School of Public Health
-     University of California at Berkeley
-     Berkeley, CA 94720
-
-     fbois@diana.lbl.gov
-
    -- Revisions -----
-     Logfile:  SCCS/s.simi.h
-    Revision:  1.10
-        Date:  14 Nov 1997
-     Modtime:  06:39:20
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -62,24 +54,25 @@ void FreeLevels (PANALYSIS panal);
 int  FreeMCLists (PVOID pData, PVOID pUserInfo);
 void FreeOneLevel (PLEVEL plevel);
 BOOL GetData (PINPUTBUF pibIn, PSTR szLex, POUTSPEC pos);
-BOOL GetGibbsSpec (PINPUTBUF pibIn, PEXPERIMENT pexp);
+BOOL GetMCMCSpec (PINPUTBUF pibIn, PEXPERIMENT pexp);
 BOOL GetIntegrate (PINPUTBUF pibIn, PINTSPEC pis);
 PSTR GetKeyword (int iKWCode);
 int  GetKeywordCode (PSTR szKeyword, PINT pfContext);
 BOOL GetListOfData (PINPUTBUF pibIn, PDATAREC pda, PSTR szLex);
 BOOL GetListOfTimes (PINPUTBUF pibIn, int nRecs, PPRINTREC *ppr, PSTR szLex);
-int  GetMCVaryParam(PINPUTBUF pibIn, PSTR szLex,
-                    PLIST plist, int n, PMCVAR pmcvar);
-int  GetMCVarySpec (PINPUTBUF pibIn, PANALYSIS panal, PSTR szLex);
+int  GetDistribParam(PINPUTBUF pibIn, PSTR szLex,
+                     PLIST plist, int n, PMCVAR pmcvar);
+int  GetDistribSpec (PINPUTBUF pibIn, PANALYSIS panal, PSTR szLex);
 int  GetMonteCarloSpec (PINPUTBUF pibIn, PANALYSIS panal, PSTR szLex);
 BOOL GetOptDSpec (PINPUTBUF pibIn, PANALYSIS  panal, PSTR szLex);
-BOOL GetOutputFile (PINPUTBUF pibIn, PSTR szLex, POUTSPEC pos);
+BOOL GetOutputFile (PINPUTBUF pibIn, PSTR szLex, PANALYSIS panal);
 BOOL GetParmMod (PINPUTBUF pibIn, PSTRLEX szLex, PSTREQN szeqn);
 BOOL GetPrint (PINPUTBUF pibIn, PSTR szLex, POUTSPEC pos);
 BOOL GetPrintStep (PINPUTBUF pibIn, PSTR szLex, POUTSPEC pos);
 int  GetSetPointsSpec (PINPUTBUF pibIn, PANALYSIS  panal, PSTR szLex);
 BOOL GetSimType (PINPUTBUF pibIn);
-BOOL GetSimulate (PINPUTBUF pibIn, PEXPERIMENT pexp);
+BOOL GetSimulate ();
+BOOL GetStartTime (PINPUTBUF pibIn, PEXPERIMENT pexp);
 BOOL GetStringArg (PINPUTBUF pibIn, PSTR *pszArg, PSTR szLex, BOOL bDelim);
 int  GetTerminator (PINPUTBUF pibIn, PSTR szLex);
 long ImFromLex (PSTR szLex);
