@@ -3,7 +3,7 @@
    written by Don Maszle
    22 November 1991
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1991-2004.  Don Maszle, Frederic Bois.  All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -19,19 +19,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-   contact
-     Frdric Bois / Don Maszle
-     BEHS, School of Public Health
-     University of California at Berkeley
-     Berkeley, CA 94720
-
-     fbois@diana.lbl.gov
-
    -- Revisions -----
-     Logfile:  SCCS/s.siminit.c
-    Revision:  1.17
-        Date:  14 Nov 1997
-     Modtime:  06:39:09
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -67,7 +59,7 @@ void GetModelInfo (PMODELINFO pmi)
 } /* GetModelInfo */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    InitIntegratorSpec
 */
 
@@ -93,27 +85,29 @@ void InitIntegratorSpec (PINTSPEC pis)
 } /* InitIntegratorSpec */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    InitOutputSpec
 */
 
 void InitOutputSpec (POUTSPEC pos)
 {
-  pos->szOutfilename    = NULL;
-  pos->pfileOut         = NULL;
-  pos->bCommandLineSpec = FALSE;
-
-  pos->nOutputs         = 0;       /* Init output list to empty */
+  pos->nOutputs         = 0;
   pos->pszOutputNames   = NULL;
-  pos->phvar = NULL;
+  pos->phvar_out        = NULL;
 
-  pos->plistPrintRecs   = NULL;    /* List of Print() records to define ... */
-  pos->plistDataRecs    = NULL;    /* List of Print() records to define ... */
+  pos->nData            = 0;
+  pos->pszDataNames     = NULL;
+  pos->phvar_dat        = NULL;
+
+  pos->plistPrintRecs   = NULL;    /* List of Print() records read in */
+  pos->plistDataRecs    = NULL;    /* List of Data() records read in */
 
   pos->pcOutputTimes    = NULL;    /* Count of output times for each var */
   pos->piCurrentOut     = NULL;    /* Index to current output for each var */
   pos->prgdOutputTimes  = NULL;    /* Array of output times for each var */
   pos->prgdOutputVals   = NULL;    /* Array of output values for each var */
+
+  pos->pcData           = NULL;
   pos->prgdDataVals     = NULL;    /* Array of data values for each var */
 
   pos->cDistinctTimes   = 0;       /* Init output times list to empty */
@@ -122,7 +116,7 @@ void InitOutputSpec (POUTSPEC pos)
 } /* InitOutputSpec */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
     InitExperiment
 
 */
@@ -135,8 +129,6 @@ void InitExperiment (PEXPERIMENT pexp, PMODELINFO pmodelinfo)
 
   pexp->dTime = 0.0; /* Current time, not used for global */
 
-  pexp->iSubject = 0;
-
   pexp->pmodelinfo = pmodelinfo; /* Same model for all experiments */
   pexp->plistParmMods = InitList(); /* List of parameter modifications */
 
@@ -146,7 +138,7 @@ void InitExperiment (PEXPERIMENT pexp, PMODELINFO pmodelinfo)
 } /* InitExperiment */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    InitMonteCarlo
 
 */
@@ -166,7 +158,7 @@ void InitMonteCarlo (PMONTECARLO pmc)
 } /* InitMonteCarlo */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    InitGibbs
 */
 
@@ -188,7 +180,7 @@ void InitGibbs (PGIBBSDATA pgd)
 } /* InitGibbs */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    InitAnalysis
 
    Initializes an analysis structure and its associated
@@ -204,11 +196,9 @@ void InitAnalysis (PANALYSIS panal)
 
   panal->bDependents = panal->bParams = FALSE;
 
-  panal->iType = AT_NOTSPECD; /* Type of analysis */
+  panal->iType = AT_DEFAULTSIM; /* Type of analysis, default to simple sim */
 
-  panal->dSeed  = SEED_DEFAULT; /* The seed for everybody */
-
-  panal->plistModelVars = InitList();
+  panal->dSeed = SEED_DEFAULT; /* The seed for everybody */
 
   panal->wContext = CN_GLOBAL; /* Begin in global context */
   panal->pexpCurrent = &panal->expGlobal;
@@ -218,9 +208,13 @@ void InitAnalysis (PANALYSIS panal)
   /* Global experiment settings */
   InitExperiment (&panal->expGlobal, &panal->modelinfo);
 
+  panal->szOutfilename    = NULL;
+  panal->pfileOut         = NULL;
+  panal->bCommandLineSpec = FALSE;
+
   /* Init all experiments NULL */
   panal->iExpts = 0;
-  for (i = 0; i < MAX_EXPERIMENTS; i++) panal->rgpExps[i] = NULL;
+  for (i = 0; i < MAX_INSTANCES; i++) panal->rgpExps[i] = NULL;
 
   InitMonteCarlo (&panal->mc); /* Monte Carlo specifications */
   InitGibbs (&panal->gd); /* Gibbs specifications */
@@ -228,7 +222,7 @@ void InitAnalysis (PANALYSIS panal)
 } /* InitAnalysis */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    InitOutputs
 
    Returns TRUE if ok, FALSE otherwise.
@@ -256,7 +250,7 @@ BOOL InitOutputs (PEXPERIMENT pexp, PINT piOut, PDOUBLE pdTout)
 } /* InitOutputs */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    InitOneOutVar
 
    Initializes information for one output to be printed.  Creates space
@@ -270,7 +264,7 @@ int InitOneOutVar (PVOID pData, PVOID pInfo)
   int i = pos->nOutputs++; /* Index of current cell */
 
   pos->pszOutputNames[i] = ppr->szOutputName;
-  pos->phvar[i]          = ppr->hvar;
+  pos->phvar_out[i]      = ppr->hvar;
   pos->pcOutputTimes[i]  = ppr->cTimes;
   pos->piCurrentOut[i]   = 0;
 
@@ -284,13 +278,10 @@ int InitOneOutVar (PVOID pData, PVOID pInfo)
 } /* InitOneOutVar */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    InitOneDataVar
 
-   Initializes information for one data specification
-   Creates space for data and check that they are associated
-   with an already created output spec and have matching number
-   of time points.
+   Initializes information for one data specification.
 */
 
 int InitOneDataVar (PVOID pData, PVOID pInfo)
@@ -298,39 +289,19 @@ int InitOneDataVar (PVOID pData, PVOID pInfo)
 
   PDATAREC pda = (PDATAREC) pData;
   POUTSPEC pos = (POUTSPEC) pInfo;
-  int i;
+  int i = pos->nData++; /* Index of current cell */
 
-  /* check that the variable referenced in the data statement
-     has a print specification */
-  i = 0;
-  while ((i < pos->nOutputs) && (pos->phvar[i] != pda->hvar))
-   i++;
-
-  /* yes, one was found */
-  if (i < pos->nOutputs) { /* n data = n times ? */
-    if (pos->pcOutputTimes[i] == pda->cData)
-      pos->prgdDataVals[i] = pda->pdData;
-    else {
-      printf ("Error: There must be as many data in the Data " 
-              "statement as times \n" 
-              "       in the Print or PrintStep statement for %s - Exiting.\n",
-              pda->szDataName);
-      exit(0);
-    }
-  }
-  else {
-    printf ("Error: The Data statement for %s must have a matching \n",
-            pda->szDataName);
-    printf ("       Print or PrintStep statement - Exiting.\n");
-    exit(0);
-  }
+  pos->prgdDataVals[i] = pda->pdData; /* point to the data read in */
+  pos->pcData[i] = pda->cData;
+  pos->phvar_dat[i] = pda->hvar;
+  pos->pszDataNames[i] = pda->szDataName;
 
   return 0;
 
 } /* InitOneDataVar */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    FindNewPoint
 
    Finds the new point.  Return FALSE if the sort is finished, i.e. no
@@ -348,7 +319,7 @@ BOOL FindNewPoint (POUTSPEC pos, PINT piPoint)
 } /* FindNewPoint */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    CreateOutputSchedule
 
    Creates a single output time array from the individual times.  This
@@ -413,53 +384,41 @@ void CreateOutputSchedule (POUTSPEC pos)
 } /* CreateOutputSchedule */
 
 
-/* -------------------------------------------------------------------w
+/* ----------------------------------------------------------------------------
    PrepareOutSpec
 
    Prepares an output spec determined by the plistPrintRecs
-   and plistDataRecs specification.
+   and plistDataRecs specifications.
    Adjusts final time eventually to match the last output time
 */
 
 BOOL PrepareOutSpec (PEXPERIMENT pexp)
 {
-  POUTSPEC pos = &pexp->os;
+  POUTSPEC pos  = &pexp->os;
   BOOL bReturn = FALSE;
+  int cDat = ListLength(pos->plistDataRecs);
   int cOut = ListLength(pos->plistPrintRecs);
-  int i;
 
   if (!cOut)
     ReportError (NULL, RE_NOOUTPUTS, (PSTR) &pexp->iExp, NULL);
+  else { /* ok, cOut is not null */
+    pos->pszOutputNames  = (PSTR *) malloc (cOut * sizeof(PSTR));
+    pos->phvar_out       = (HVAR *) malloc (cOut * sizeof(HVAR));
+    pos->pcOutputTimes   = InitiVector (cOut);
+    pos->piCurrentOut    = InitiVector (cOut);
+    pos->prgdOutputTimes = InitpdVector (cOut);
+    pos->prgdOutputVals  = InitpdVector (cOut);
 
-  else {
-    pos->pszOutputNames = (PSTR *) malloc (cOut * sizeof(PSTR));
-    pos->phvar          = (HVAR *) malloc(cOut * sizeof(HVAR));
-    pos->pcOutputTimes  = InitiVector (cOut);
-    pos->piCurrentOut   = InitiVector (cOut);
-    pos->prgdOutputTimes= InitpdVector (cOut);
-    pos->prgdOutputVals = InitpdVector (cOut);
-    pos->prgdDataVals   = InitpdVector (cOut);
-
-    if (pos->pszOutputNames  == NULL || pos->phvar == NULL          || 
-        pos->pcOutputTimes   == NULL || pos->piCurrentOut == NULL   ||    
-        pos->prgdOutputTimes == NULL || pos->prgdOutputVals == NULL ||  
-        pos->prgdDataVals    == NULL)
+    if (pos->pszOutputNames  == NULL || pos->phvar_out == NULL    || 
+        pos->pcOutputTimes   == NULL || pos->piCurrentOut == NULL ||    
+        pos->prgdOutputTimes == NULL || pos->prgdOutputVals == NULL)
       ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "PrepareOutSpec()", NULL);
-
     else {
-      pos->nOutputs = 0;  /* Current cell counter in callback */
+      pos->nOutputs = 0; /* Current cell counter in callback */
       ForAllList (pos->plistPrintRecs, InitOneOutVar, (PVOID) &pexp->os);
-      pos->nOutputs = cOut;  /* Set count of outputs for real */
-
-      /* initialize the prgdDataVals array with the data spec list or
-         to null.
-         This must be called here, after initializing the outputs to
-         allow for matching of outputs and data */
-      for (i = 0; i < cOut; i++) pos->prgdDataVals[i] = NULL;
-      ForAllList (pos->plistDataRecs, InitOneDataVar, (PVOID) &pexp->os);
+      pos->nOutputs = cOut; /* Set count of outputs for real */
 
       FreeList (&pos->plistPrintRecs, NULL, TRUE);
-      FreeList (&pos->plistDataRecs, NULL, TRUE);
 
       CreateOutputSchedule (pos); /* Create single output time list */
 
@@ -467,12 +426,33 @@ BOOL PrepareOutSpec (PEXPERIMENT pexp)
       pexp->dTfinal = pos->rgdDistinctTimes[pos->cDistinctTimes - 1];
 
       if (pexp->dTfinal == pexp->dT0) {
-        printf ("\nError: starting and final times are equal in Experiment %d "
+        printf ("\nError: starting and final times are equal in Simulation %d "
                 "- Exiting.\n\n", pexp->iExp);
         exit(0);
       }
 
       bReturn = TRUE;
+
+    } /* else */
+  } /* else */
+
+  if (!cDat)
+    ; /* cDat is null, but it's allowed */
+  else { /* ok, cDat is not null */
+    pos->prgdDataVals = InitpdVector (cDat);
+    pos->pcData       = InitiVector (cDat);
+    pos->pszDataNames = (PSTR *) malloc (cDat * sizeof(PSTR));
+    pos->phvar_dat    = (HVAR *) malloc (cDat * sizeof(HVAR));
+
+    if (pos->prgdDataVals == NULL || pos->phvar_dat == NULL ||
+        pos->pszDataNames == NULL || pos->pcData    == NULL)
+      ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "PrepareOutSpec()", NULL);
+    else {
+      pos->nData = 0; /* Current cell counter in callback */
+      ForAllList (pos->plistDataRecs, InitOneDataVar, (PVOID) &pexp->os);
+      pos->nData = cDat;  /* Set count of data for real */
+
+      FreeList (&pos->plistDataRecs, NULL, TRUE);
     } /* else */
   } /* else */
 
@@ -481,19 +461,27 @@ BOOL PrepareOutSpec (PEXPERIMENT pexp)
 } /* PrepareOutSpec */
 
 
+/* ----------------------------------------------------------------------------
+   PrintOutSpec
+
+*/
+
 BOOL PrintOutSpec (PEXPERIMENT pexp)
 {
   POUTSPEC pos = &pexp->os;
   int j, i, cOut = pos->nOutputs;
 
   printf ("%d Outputs:\n", cOut);
+
   for (i = 0; i < cOut; i++) {
-    printf ("  %#0x  %s: ", pos->phvar[i], pos->pszOutputNames[i]);
+    printf ("  %#0x  %s: ", pos->phvar_out[i], pos->pszOutputNames[i]);
     for (j = 0; j < pos->pcOutputTimes[i]; j++)
       printf ("%g ", pos->prgdOutputTimes[i][j]);
     printf ("\n");
-  } /* for */
+  } 
+
   return 0;
+
 } /* PrintOutSpec */
 
 /* End */
