@@ -1,6 +1,6 @@
 /* simmonte.c
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1993-2004.  Don Maszle, Frederic Bois.  All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -16,19 +16,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-   contact
-     Frdric Bois / Don Maszle
-     BEHS, School of Public Health
-     University of California at Berkeley
-     Berkeley, CA 94720
-
-     fbois@diana.lbl.gov
-
    -- Revisions -----
-     Logfile:  SCCS/s.simmonte.c
-    Revision:  1.26
-        Date:  14 Nov 1997
-     Modtime:  06:39:09
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -49,7 +41,7 @@
 #include "simmonte.h"
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    SetParms
 
    sets the parameters in the rghvar array to the values in the rgdParm
@@ -66,7 +58,7 @@ void SetParms (long cParms, HVAR *rghvar, double *rgdParm)
 } /* SetParms */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    SetParmsLog
 
    sets the parameters in the rghvar array to the log-transformed
@@ -83,7 +75,24 @@ void SetParmsLog (long cParms, HVAR *rghvar, double *rgdParm)
 } /* SetParmsLog */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
+   SetParmsExp
+
+   sets the parameters in the rghvar array to the exp-transformed
+   values in the rgdParm array.
+*/
+
+void SetParmsExp (long cParms, HVAR *rghvar, double *rgdParm)
+{
+  long i;
+
+  for (i = 0; i < cParms; i++)
+    SetVar (rghvar[i], exp(rgdParm[i]));
+
+} /* SetParmsExp */
+
+
+/* ----------------------------------------------------------------------------
    CalculateOneMCParm
 
    Callback function for CalculateMCParms.
@@ -119,6 +128,10 @@ int CalculateOneMCParm (PMCVAR pMCVar)
       pMCVar->dVal = NormalRandom (dParm1, dParm2);
       break;
 
+    case MCV_HALFNORMAL:
+      pMCVar->dVal = abs(NormalRandom (dParm1, dParm2));
+      break;
+
     case MCV_LOGNORMAL:
       pMCVar->dVal = LogNormalRandom (dParm1, dParm2);
       break;
@@ -172,6 +185,10 @@ int CalculateOneMCParm (PMCVAR pMCVar)
       pMCVar->dVal = InvGGammaRandom (dParm1, dParm2);
       break;
 
+    case MCV_TRUNCINVGGAMMA:
+      pMCVar->dVal = TruncInvGGammaRandom (dParm1, dParm2, dMin, dMax);
+      break;
+
     case MCV_POISSON:
       pMCVar->dVal = PoissonRandom (dParm1);
       break;
@@ -187,7 +204,7 @@ int CalculateOneMCParm (PMCVAR pMCVar)
 } /* CalculateOneMCParm */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    CalcMCParms
 
    calculates random parameters for a Monte Carlo variation.
@@ -216,7 +233,7 @@ void CalcMCParms (PMONTECARLO pMC, double rgParms[], long iStart)
 } /* CalcMCParms */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    InitSetPoints
 
    Openn and reads the header of the SetPoint file containing the
@@ -248,7 +265,7 @@ BOOL InitSetPoints (PMONTECARLO pMC)
 } /* InitSetPoints */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    ReadSetPoints
 
    Reads set points from a file for this run.
@@ -300,7 +317,7 @@ Exit_ReadSetPoints:
 } /* ReadSetPoints */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    GetMCMods
 
    Calculates random parameter variations or reads a new set of
@@ -336,11 +353,11 @@ BOOL GetMCMods (PANALYSIS panal, double rgParms[])
 } /* GetMCMods */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    SetParents
 
    FB 21/07/97: For each Monte Carlo variable, if there are parents referenced
-   in hParm for some of the distributionla parameters, pdParms are set to point
+   in hParm for some of the distributional parameters, pdParms are set to point
    to the parent's dVals. If there is no parent, pdParms already point to their
    own dParms and nothing is done.
 
@@ -356,7 +373,7 @@ void SetParents (PMONTECARLO pMC, long iStart)
   for (i = iStart; i < pMC->nParms; i++) { /* for each pMCVar considered */
     pMCVar1 = pMC->rgpMCVar[i];
     for (j = 0; j < 4; j++) { /* for each of its distrib. param */
-      if (pMCVar1->hParm[j] != 0) { 
+      if (pMCVar1->iParmType[j] == MCVP_PARM) { 
         /* if there is a parent, find it, but parents must appear before 
            current pMCVar */
         bFound = FALSE;
@@ -370,8 +387,8 @@ void SetParents (PMONTECARLO pMC, long iStart)
         }
         if (!bFound) { /* oops, parent not found, error */
           printf ("\n"
-                  "Error: parents must appear in Distrib statements before\n"
-                  "       childrens when creating sampling dependencies - Exiting.\n\n");
+                  "Error: parents must be declared before childrens when\n"
+                  "       creating sampling dependencies - Exiting.\n\n");
           exit(0);
         }
       }
