@@ -3,7 +3,7 @@
    written by Don Maszle
    22 November 1991
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   Copyright (c) 1991-2004.  Don Maszle, Frederic Bois.  All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -19,19 +19,11 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
-   contact
-     Frdric Bois / Don Maszle
-     BEHS, School of Public Health
-     University of California at Berkeley
-     Berkeley, CA 94720
-
-     fbois@diana.lbl.gov
-
    -- Revisions -----
-     Logfile:  SCCS/s.simo.c
-    Revision:  1.26
-        Date:  14 Nov 1997
-     Modtime:  06:39:10
+     Logfile:  %F%
+    Revision:  %I%
+        Date:  %G%
+     Modtime:  %U%
       Author:  @a
    -- SCCS  ---------
 
@@ -62,7 +54,7 @@ void SaveOutputs (PEXPERIMENT pexp, PDOUBLE pdTout)
 {
   #define SO_EPSILON (1e-100) /* Smaller values are zeroed  */
 
-  static     PDOUBLE  rgdInterpStates, rgdInterpDeriv;
+  static     PDOUBLE rgdInterpStates, rgdInterpDeriv;
   int        i, j, index;
   PMODELINFO pmod = pexp->pmodelinfo;
   POUTSPEC   pos = &pexp->os;
@@ -91,13 +83,13 @@ void SaveOutputs (PEXPERIMENT pexp, PDOUBLE pdTout)
         && *pdTout == pos->prgdOutputTimes[i][pos->piCurrentOut[i]]) {
       double dTmp = 0;
 
-      if (IsModelVar(pos->phvar[i]))  /* Use interp'd model value */
-        dTmp = rgdInterpStates[ ModelIndex(pos->phvar[i])];
+      if (IsModelVar(pos->phvar_out[i]))  /* Use interp'd model value */
+        dTmp = rgdInterpStates[ ModelIndex(pos->phvar_out[i])];
 
       else { /* Use current parm/input value */
-        index = HINDEX(pos->phvar[i]);
+        index = HINDEX(pos->phvar_out[i]);
         j = vrgInputs[index].iDoseCur;
-        if (IsInput(pos->phvar[i]) && 
+        if (IsInput(pos->phvar_out[i]) && 
             (vrgInputs[index].iType == IFN_SPIKES)) {
  
           /* FB 11/6/97 */
@@ -109,7 +101,7 @@ void SaveOutputs (PEXPERIMENT pexp, PDOUBLE pdTout)
 
         }
         else
-          dTmp = GetVarValue (pos->phvar[i]);
+          dTmp = GetVarValue (pos->phvar_out[i]);
       }
 
       if (fabs(dTmp) < SO_EPSILON) /* Avoid silly little numbers  */
@@ -132,11 +124,14 @@ void SaveOutputs (PEXPERIMENT pexp, PDOUBLE pdTout)
 
 void NextOutputTime (PEXPERIMENT pexp, PDOUBLE pdTout, PINT piOut)
 {
-  if (pexp->dTime < pexp->dTfinal)
-    if (++*piOut < pexp->os.cDistinctTimes)
+  if (pexp->dTime < pexp->dTfinal) {
+    if (++*piOut < pexp->os.cDistinctTimes) {
       *pdTout = pexp->os.rgdDistinctTimes[*piOut];
-    else
+    }
+    else {
       *pdTout = pexp->dTfinal;
+    }
+  }
 
 } /* NextOutputTime */
 
@@ -209,8 +204,8 @@ int OpenMCFiles (PANALYSIS panal)
   PMONTECARLO pmc = &panal->mc;
 
   /* Use command line spec if given */
-  if (panal->expGlobal.os.bCommandLineSpec)
-    pmc->szMCOutfilename = panal->expGlobal.os.szOutfilename;
+  if (panal->bCommandLineSpec)
+    pmc->szMCOutfilename = panal->szOutfilename;
   else 
     if (!(pmc->szMCOutfilename))  /* Default if none given */
       pmc->szMCOutfilename = vszDefMCOutFilename;
@@ -281,25 +276,23 @@ void WriteMCOutput (PANALYSIS panal, PMCPREDOUT pmcpredout)
 
 void WriteNormalOutput (PANALYSIS panal, PEXPERIMENT pexp)
 {
-  int i, j;
-  PFILE pfile;
-  POUTSPEC posGlo, pos;
+  long     i, j;
+  PFILE    pfile;
+  POUTSPEC pos;
 
   if (!panal) return;
 
-  posGlo = &panal->expGlobal.os;
   pos = &pexp->os;
 
-  if (!posGlo->szOutfilename)
-    posGlo->szOutfilename = vszDefOutFilename;
+  if (!panal->szOutfilename)
+    panal->szOutfilename = vszDefOutFilename;
 
-  if (!(posGlo->pfileOut))
-    if (!(posGlo->pfileOut = fopen (posGlo->szOutfilename, "w")))
-      ReportError (NULL, RE_CANNOTOPEN | RE_FATAL,
-               posGlo->szOutfilename, NULL);
+  if (!(panal->pfileOut))
+    if (!(panal->pfileOut = fopen (panal->szOutfilename, "w")))
+      ReportError (NULL, RE_CANNOTOPEN | RE_FATAL, panal->szOutfilename, NULL);
 
-  pfile = posGlo->pfileOut;
-  fprintf (pfile, "Results of Experiment %d\n\n", pexp->iExp);
+  pfile = panal->pfileOut;
+  fprintf (pfile, "Results of Simulation %d\n\n", pexp->iExp);
 
   /* Vertical output:  Formatted  Time1    Out_Var1  Out_Var2 ... */
 
